/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.context
 * Contextual menus
 *
 * Example:
 * (code)
 *     var contextMenu1 = new OS.context({
 *         'container': 'desktop',
 *         'list': [{
 *                 'name': 'Test 1',
 *                 'action': function(){alert('1');},
 *                 'icon': 'themes/default/images/icons/kblackbox16.png'
 *             },{
 *                 'name': 'Test 2',
 *                 'action': function(){alert('2');}
 *             },{
 *                 'separator': true
 *             },{
 *                 'name': 'Test 3',
 *                 'disabled': true
 *             },
 *         ]
 *     });
 * (end)
 **********************************************/
OS.context = new Class({
    /**********************************************
     * Properties: Properties
     **********************************************/
    'Implements': Options,
    'options': {
        'container': 'os',
        'list': {}
    },
    'elements': {
        'menu': false,
        'container': false,
        'container_real': $('os')
    },
    
    /**********************************************
     * Method: initialize
     **********************************************/
    'initialize': function(options){
        // Merge passed options with default
        this.setOptions(options);
        // Create context menu
        this.setContainer();
        this.create();
        this.add();
    },
    
    /**********************************************
     * Method: setContainer
     **********************************************/
    'setContainer': function(){
        if($type(this.options.container=='string')) {
            this.elements.container = $(this.options.container);
        }
        else if($type(this.options.container=='element')) {
            this.elements.container = this.options.container;
        }
    },

    /**********************************************
     * Method: create
     * Create the menu Element
     **********************************************/
    'create': function(){
        var menu = new Element('ul', {
            'class': 'contextmenu'
        });
        var list = this.options.list;
        var element, name, action, icon;
        var o = this;
        list.each(function(elem){
            // Separator
            if(elem.separator){
                element = new Element('li', {
                    'class': 'separator'
                });   
            }
            // List item
            else {
                element = new Element('li', {
                    'html': elem.name,
                    'styles': {
                        'background-image': 'url('+(elem.icon?elem.icon:'')+')',
                        'background-position': '2px 50%',
                        'background-repeat': 'no-repeat'
                    },
                    'events': {
                        'mousedown': function(e){
                            e.stopPropagation();
                            if($type(elem.action)=='function' && !elem.disabled){
                                elem.action();
                            }
                            if(!elem.disabled){
                                o.remove();
                            }
                        }
                    }
                });
                if(elem.disabled){
                    element.addClass('disabled');
                }
            }
            element.inject(menu);
        });
        this.elements.menu = menu;
    },
    
    /**********************************************
     * Method: add
     * Add the context menu event to the container
     **********************************************/
    'add': function(){
        var o = this;
        var pos1,pos2;
        if(o.elements.container){
            o.elements.container.addEvents({
                'contextmenu': function(e){
                    e.preventDefault();
                    e.stopPropagation();
                    pos1 = e.client.x;
                    pos2 = e.client.y;
                    o.elements.menu.set('styles', {
                        'position': 'absolute',
                        'left':pos1,
                        'top':pos2
                    })
                    o.elements.menu.inject(o.elements.container_real);
                }
            });
            o.elements.container_real.addEvents({
                'mousedown': function(){
                    o.remove();
                }
            });
        }
    },
    
    /**********************************************
     * Method: remove
     * Remove the context menu
     **********************************************/
    'remove': function(){
        this.elements.menu.dispose();
    },
    
    /**********************************************
     * Method: destroy
     * Destroy event listener and remove menu
     **********************************************/
    'destroy': function(){
        //FIXME: doesn't work
        this.remove();
        this.elements.container_real.removeEvent('mousedown'); // Dangerous !
        this.elements.container.removeEvent('contextmenu');
    }
});
