/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

             v. 1.3
        Web Operating System
*/
/*global $, $$, $defined, $random, $time, $type, Class, Drag, Element, Request, clearInterval, clearTimeout, document, navigator, os, session_id, setInterval, setTimeout, user, window*/
/**********************************************
 * Class: OS.core
 * Main class that:
 * - handle modules
 * - handle asyncrone requests
 * - creates the desktop
 * - provides some utilities as base64 decode/encode, etc
 **********************************************/
OS.core = new Class({
/**********************************************
 * Properties: Properties
 *
 * agent - user agent
 * desktop - desktop Element
 * dw - desktop width
 * dh - desktop height
 * hour_cycle - Update cycle for the hour, in milliseconds
 * icons - icons management
 * new_window_pos - new position, used in setWinPosQuad
 * path - location of this script
 * pi - 15 after comma PI value
 * selected_icon - ID of current selected icon
 * selected_window - ID of current selected window
 * system - Clock Element
 * system_info - Element for logs
 * time - Standard Time
 * user - Username
 * windows - Windows management
 * zindex_id - Next z-index for windows
 * 
 **********************************************/ 
    'agent': navigator.userAgent,
    'user': user,
    'dw': 0,
    'dh': 0,
    'hour_cycle': 10000,
    'path': '',
    'time': new Date(),
    'pi': Math.PI,
    'os': $('os'),
    'desktop': $('desktop'),
    'system': $('system'),
    'system_info': $('system_info'),
    'windows': {
        'info': [],
        'xref': [],
        'unmax': [],
        'parents': []
    },
    'icons': {
        'info': [],
        'xref': [],
        'parents': [],
        'container': []
    },
    'selected_icon':'',
    'selected_window':'',
    'new_window_pos': 0,
    'zindex_id': 2,
    'location_hash': '',

/**********************************************
 * Constructor: initialize
 * Fired at instanciation in init.js
 **********************************************/
    'initialize': function()
    {
        this.path = this.dirname(window.location.pathname);        
        
        //Get Icons and Applications
        this.sendRequest('Kernel/Server/syscall.php', {
            'post': 'req=createDesktop',
            'onComplete': function(){
                OS.createDesktop();
            }
        });
    },

/**********************************************
 * Method: createDesktop
 * Desktop creation (menu, system, desktop info,
 * desktop icons)
 **********************************************/
    'createDesktop': function()
    {
        this.dw = this.desktop.offsetWidth;
        this.dh = this.desktop.offsetHeight;
        this.windows.info.push([this.dw, this.dh, 0, 0]);
        this.windows.xref.push('desktop');
        this.windows.parents.push('os');
        this.hour();
        
        // Events
        window.addEvent('resize', function() {
            OS.dw = OS.desktop.offsetWidth;
            OS.dh = OS.desktop.offsetHeight;
            OS.windows.info[0] = [OS.dw, OS.dh, 0, 0];
        });
        this.desktop.set('events', {
            'dblclick': function(e){
                e.preventDefault();
            }
        });
        this.os.set('events', {
            'dblclick': function(e){
                e.preventDefault();
            }
        });
        
        //check location hash change (not efficient)
        var locationCheck = setInterval(function(){
            var state = OS.getLocationState();
            // if state changed
            if(state && state.w!=OS.location_hash.w){
                // apply new state
                OS.location_hash = state;
                if(!OS.updateZindex(state.w)){
                    OS.startApp(state.a);
                }
            }
        }, 250);
        
        // Application at startup (TODO: startup + last session)
        //this.startApp('irc');
        //this.startApp('programs');
        
        // Widgets: Menu, taskbar, ..
        var menu = new OS.widget.init.menu({
            'name':'Menu',
            'movable': true
        });
    },

/**********************************************
 * Group: Request
 **********************************************/
/**********************************************
 * Method: sendRequest
 * Send a request to PHP file and Wait for
 * answer(s) to handle. POST Data only !
 *
 * See <OS.buildRequest>
 * 
 * Parametres:
 *     url - url
 *     options - see Options
 *
 * Options:
 *  post - post data
 *  id - id of the request, Default: current timestamp
 *  onComplete - function to execute when request complete (on succes or failure)
 *  onFailure - function to execute when the request fails
 *  onSuccess - function to execute when the request succed
 *
 * Examples:
 *  *Update the content of a DIV*
 *  - JS Side
 *  (start code)
 *    OS.sendRequest('programs/test/test.php', {
 *        post: 'foo=bar&someVar=someValue'
 *    });
 *  (end)
 *  - PHP Side
 *  (start code)
 *    $someVar = $_POST['someVar'];
 *    $request[] = array('window_update', array('id_of_DIV_to_update', 'content_to_put_in'));
 *    echo os::buildRequest($request);
 *  (end)
 *  
 *  *Chained requests*
 *  - JS Side
 *  (start code)
 *    OS.sendRequest('programs/test/test.php', {
 *        post: 'foo=bar'
 *    });
 *  (end)
 *  - PHP Side
 *  (start code)
 *    $request[] = array('window_update', array('id1', 'content1', 'before'));
 *    $request[] = array('window_update', array('id2', 'content2', 'top'));
 *    $request[] = array('exec', 'OS.infoWindow("Information", "test_hw"+$time(), {"content": "Hello World !"});');
 *    echo os::buildRequest($request);
 *  (end)
 **********************************************/    
    'sendRequest': function(url, options)
    {
        var onComplete = options.onComplete || function(){};
        var onSuccess = options.onSuccess || function(){};
        var onFailure = options.onFailure || false;
        var post = options.post || '';
        var id = options.id || $time();
        
        var req = new Request(
        {
            'url': url+'?session_id='+session_id+'&id='+id,
            'method': 'post',
			'evalScripts': false,
            'onSuccess': function(res){
                OS.handleRequest(res);
                onSuccess(res);
            },
            'onFailure': function(err){
                var response = err.responseText;
                if(onFailure) {
                    onFailure(err);
                }
                else {
                    OS.systemInfo('<strong>Error: </strong>'+response);
                }
            },
            'onComplete': function(res){
                onComplete(res);
            }
        }).send(post);
    },

/**********************************************
 * Method: handleRequest
 * 
 * Parametres:
 *     json - json
 **********************************************/
    'handleRequest': function(json)
    {
        var res = JSON.decode(json, true);
        if(res!=null){
            var result = OS.jsonDb(res), type, data;
            result.each(function(res)
            {
                type = res.type;
                data = res.data;
                if(data)
                {
                    switch(type)
                    {
                        case 'apps':
                            OS.loadModules(data);
                            break;
                        
                        case 'icons':
                            OS.addIcons(data);
                            break;
                        
                        case 'exec':
                            var func = function(){
                                window.eval(data);
                                return true;
                            };
                            var exec = func.attempt();
                            if(exec==null){
                                OS.infoWindow('Alert !', $time()+'_error', {'content': 'Invalid Javascript !'});
                            }
                            break;
                        
                        case 'window_update':
                            var item = $(data[0]);
                            var html = data[1];
                            var pos = data[2];
                            content = new Element('span', {
                                'html': html
                            });
                            if(pos) {
                                content.inject(item, pos);
                            }
                            else {
                                item.set('html', html);
                            }
                            break;
                        
                        case 'window_info':
                            var title = data[0];
                            var options = data[1];
                            OS.infoWindow(title, $time()+'_request', options);
                            break;
                        
                        case 'window_create':
                            var id = data[0];
                            var title = data[1];
                            var options = data[2];
                            OS.createWindow(title, id, options);
                            break;
                        
                        case 'window_remove':
                            var id = data;
                            OS.removeWindow(id);
                            break;
                        
                        default:
                            OS.systemInfo('<strong>Type error: </strong>'+type);
                    }
                }
                return true;
            });
        }
        else{
            OS.systemInfo('<strong>Request error: </strong>'+json);
        }
        return false;
    },

/**********************************************
 * Method: dbJson
 * Unmatch cols with rows to compressed JSON
 * 
 * Parameters:
 *  cols - columns
 *  array - array
 **********************************************/
'dbJson': function(cols, array)
{
    var out = {'cols': cols, 'rows': []};
    var row;
    array.each(function(app, i)
    {
        if(app[cols[0]])
        {
            row = [];
            cols.each(function(col, j)
            {
                row.push(app[col]);
            });
            out.rows.push(row);
        }
    });
    return JSON.encode(out);
},

/**********************************************
 * Method: addIcons
 **********************************************/    
    'addIcons': function(json, xdec)
    {
        var files = this.jsonDb(json);
        files.each(function(file){
            if(file.name && file.command)
            {
                OS.createIcon(file.id, {
                    'label': file.name,
                    'parent': file.parent,
                    'container': file.container,
                    'icon': file.icon,
                    'cmd': [file.command, file.context],
                    'x': (xdec) ? xdec : false,
                    'y': 1
                });
            }
        });
    },

/**********************************************
 * Group: Utilities
 **********************************************/
/**********************************************
 * Method: changeLocationState
 * Change the address bar hash and browser history.
 *
 * Parameters:
 *   location - object that describe the application state
 **********************************************/    
    'changeLocationState': function(loc)
    {
        location.hash = OS.encode64(JSON.encode(loc));
    },

/**********************************************
 * Method: getLocationState
 * Get location state object
 **********************************************/    
    'getLocationState': function()
    {
        return JSON.decode(OS.decode64(location.hash.substr(1)), true);
    },

/**********************************************
 * Method: encode64
 **********************************************/  
    'encode64': function(input) {  
        var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        var output = "";
        var chr1, chr2, chr3;
        var enc1, enc2, enc3, enc4;
        var i = 0;
        do {
            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);
            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;
            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }
            output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2) + 
            keyStr.charAt(enc3) + keyStr.charAt(enc4);
        } while (i < input.length);
    return output;
},

/**********************************************
 * Method: decode64
 **********************************************/  
    'decode64': function(input) {
        var keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        var output = "";
        var chr1, chr2, chr3;
        var enc1, enc2, enc3, enc4;
        var i = 0;
        // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
        input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
        do {
            enc1 = keyStr.indexOf(input.charAt(i++));
            enc2 = keyStr.indexOf(input.charAt(i++));
            enc3 = keyStr.indexOf(input.charAt(i++));
            enc4 = keyStr.indexOf(input.charAt(i++));
            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;
            output = output + String.fromCharCode(chr1);
            if (enc3 != 64) {
                output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
                output = output + String.fromCharCode(chr3);
            }
        } while (i < input.length);
        return output;
    },


/**********************************************
 * Method: showLoader
 * Show loader icon in an Element as background
 **********************************************/    
    'showLoader': function(elem)
    {
        elem.set('styles', {
            'background-image': 'url("themes/default/images/loader.gif")',
            'background-repeat': 'no-repeat',
            'background-position': '5px 5px'
        });
    },
    
/**********************************************
 * Method: hideLoader
 * Reset background image, position.
 **********************************************/    
    'hideLoader': function(elem)
    {
        elem.set('styles', {
            'background-image': '',
            'background-repeat': '',
            'background-position': ''
        }); 
    },

/**********************************************
 * Method: squareIntersect
 * Squares Intersection
 *
 * Parametres:
 *     c1 - array of information in the windowInformation format: [width, height, posx, posy]
 *     c2 - same as c1
 **********************************************/
    'squareIntersect': function(c1, c2)
    {
        if ( c2[0] > c1[2] || c2[2] < c1[0] || c2[1] > c1[3] || c2[3] < c1[1] ) { return(false); } 
        else { return(true); }
    },

/**********************************************
 * Method: hour
 **********************************************/    
    'hour': function()
    {
        this.time = new Date();
        var minutes = this.time.getMinutes();
        if(minutes < 10) {minutes = '0'+minutes;}
        this.system.set('html', this.time.getHours()+":"+minutes);
        setTimeout(function(){
            OS.hour();
        }, this.hour_cycle);
    },

/**********************************************
 * Method: restart
 **********************************************/
    'restart': function()
    {
        OS.infoWindow('Restart Kernel !', 'kernel_restart_question', {
            'content': 'Do you want to restart the OS ?<br/><strong>All unsaved data will be lost !</strong>',
            'actions': {
                'yes': function(){
                    $('desktop').empty();
                    $('taskbar').empty();
                    $('system_info').empty();
                    OS.sendRequest('Kernel/Server/syscall.php', {
    			    	'post': 'req=restartKernel',
                        'onComplete': function(){
                            source = JSON.encode(OS.modules.list);
                            OS.source('Kernel/init.js');
                        }
        			});
                },
                'no': function(){}
            },
            'childOf': 'cli'
        });
    },

/**********************************************
 * Method: clone
 * Clone an object or array
 * From Prototype JS
 **********************************************/
    'clone': function(array)
    {
        var newObj = (this instanceof Array) ? []:{};
        for (var i in array)
        {
            if (i!=='clone')
            {
                if (array[i] && typeof array[i]==="object")
                {
                    newObj[i] = OS.clone(array[i]);
                }
                else {newObj[i] = array[i];}
            }
        } return newObj;
    },

/**********************************************
 * Method: isNotPresent
 *
 * Parameters:
 *     src - Associative array/Object
 *     id - key
 **********************************************/
    // src=Array, id=String
    'isNotPresent': function(src, id) {
        return src.every(function(item)
        {
            return item.id!==id;
        });
    },

/**********************************************
 * Method: unset
 * Unset an element in an array and preserve indexes
 **********************************************/
    'unset': function(array, index)
    {
        var output=[];
        array.each(function(value, i)
        {
            if(i!==index)
            {
                output[i] = value;
            }
        });
        return output;
    },

/**********************************************
 * Method: systemInfo
 **********************************************/
    'systemInfo': function(txt)
    {
        var info = new Element('span', {
            'html': txt
        });
        info.inject(this.system_info, 'top');
    },

/**********************************************
 * Method: basename
 * Returns the last filename or directory name
 * of a path.
 *
 * Parameters
 *  path - path as string, backslash must be escaped
 *
 * Returns
 *  last filename or dirname
 **********************************************/
    'basename': function(path)
    {
        var re = path.match(/[\/|\\]([^\\\/]+)$/);
        return re ? re[1] : path;
    },

/**********************************************
 * Method: dirname
 * Strip out the basename of a string. Returns the path
 * to last file/directory. (Unlike PHP's dirname)
 *
 * Parameters:
 *  path - path to file or directory
 *
 * Returns:
 *  path to file/directory without last file/directory
 **********************************************/
    'dirname': function(path)
    {
        var last = this.basename(path);
        return path.substring(0, path.length - last.length);
    },

    'formatc': function(c) // EasterEgg
    {
        if(!c) {c='C:\\';}
        var easterEgg;
        var easy = function(){
            OS.infoWindow('Ahahah !', 'easter_egg'+$time(), {
                'content': '<strong>Not that easy huh :)</strong>',
                'actions': {'sorry': function(){ OS.formatc(c); }},
                'childOf': 'cli',
                'onClose': function(){
                    easy();
                }
            });
        };
        OS.infoWindow('Alert !', 'easter_egg'+$time(),{
            'content':'<strong>Do you really want to format '+c+' ?</strong>',
            'childOf': 'cli',
            'onClose': function(){
                easy();
            },
            'actions': {'Yes, sure': function(){
                OS.createWindow(
                    'Disk management',
                    'aprils_fool_format',{
                        'content':'<h2> Format '+c+' </h2><img src="themes/default/images/loading.gif"/><br />',
                        'width': 270,
                        'height': 80,
                        'posx': (OS.dw/2)-135,
                        'posy': (OS.dh/2)-50,
                        'onClose': function(){
                            clearTimeout(easterEgg);
                            OS.removeAllWindows();
                        }
                    }
                );
                var fool = function(){
                    var posx = $random(0, OS.dw-169);
                    var posy = $random(0, OS.dh-42);
                    OS.infoWindow('Alert !', 'aprils_fool_format_alert_'+$time(),{
                        'content':'<b>System failure in 0x'+$time()+'</b>',
                        'posx': posx,
                        'posy': posy,
                        'modal': false
                    });
                    OS.updateZindex('aprils_fool_format');
                    easterEgg = setTimeout(function(){
                        fool();
                    }, 300);
                };
                fool();
            }, 'No, please': function(){OS.formatc(c);}},
            'btn_default': 'no, i\'m not a fool'
        });
    },

/**********************************************
 * Group: Modules
 **********************************************/
/**********************************************
 * Method: buildModulesList
 * Request the server to rebuild modules list (apps.pb file)
 *
 * Properties:
 *  msg - message to show when asking to rebuild modules list. (default: Do you want to rebuild modules list ?)
 *  modal - if attached to a window, and want it modal, specify window id (default: false)
 **********************************************/
'buildModulesList': function(msg, modal)
{
    msg = msg ? msg : 'Do you want to rebuild modules list ?';
    modal = modal ? modal : false;
    this.infoWindow('Build modules list', 'build_modules_list', {
        'modal': modal,
        'childOf': modal,
        'content': msg,
        'actions': {
            'yes': function(){
                OS.sendRequest('Kernel/Server/syscall.php', {
                    'post': 'req=buildModulesList'
                });
            },
            'no': function(){}
        }
    });
},

/**********************************************
 * Method: loadModules
 * Create a list of installed modules and load them
 *
 * Parameters:
 *    json - Programs objects array in jsonDb format
 **********************************************/
    'loadModules': function(json)
    {
        var modules_to_load = [];
        if(json.rows.length>0)
        {
            this.modules.list = this.jsonDb(json);
            this.modules.list.each(function(mod, i)
            {
                // Load modules
                if(mod.state==='ii') {
                    modules_to_load.push(mod);
                }
            });
            if(modules_to_load.length>0) {
               this.loadModulesR(modules_to_load);
            }
        }
        else
        {
            this.buildModulesList('<b>No modules were found.</b><br/>Do you want to rebuild the database ?');
        }
    },
    

/**********************************************
 * Method: loadModulesR
 * Parallely load modules files JS then install them
 *
 * Parameters:
 *    mods - Modules path array
 **********************************************/
    'loadModulesR': function(mods)
    {
        var fn = function(){};
        var len = mods.length;
        for(var i=0; i<len; i++){
            if(i==(len-1)){
                fn = OS.installModules;
            }
            OS.source(mods[i].path, fn);
        }
    },

/**********************************************
 * Method: installModules
 * Installation des modules contenus dans OS.modules.init
 **********************************************/
    'installModules': function()
    {
        var modules_merged = [];
            
        OS.modules.init.each(function(module){
            var generic_class = new Class(OS.clone(OS.generic_module));
            var Module_class = generic_class.implement(module.init);
            var module_info = new Module_class();
            
            // Sub-module
            if(module_info.childOf!=='desktop' &&
               OS.isNotPresent(OS.modules.init, module_info.childOf) &&
               OS.isNotPresent(OS.modules.info, module_info.childOf))
            {
                var Container_class = new Class(OS.clone(OS.generic_module));
                Container_class = Container_class.implement(OS.clone(OS.container_module));
                modules_merged[module_info.childOf] = Container_class.implement(
                {
                    'id': module_info.childOf,
                    'label': module_info.childOf.capitalize()
                });
                
                // Stockage de quelques informations du sous-module
                var container_module_info = new Container_class();
                OS.modules.info.push({
                    'id': container_module_info.id,
                    'label': container_module_info.label,
                    'childOf': container_module_info.childOf,
                    'icon': container_module_info.icon,
                    'path': container_module_info.path
                });
            }
            
            // Copie de la class etendue du module
            modules_merged[module.id] = Module_class;
            
            // Stockage de quelques informations du module
            if(OS.isNotPresent(OS.modules.info, module.id))
            {
                OS.modules.info.push({
                    'id': module.id,
                    'label': module_info.label,
                    'childOf': module_info.childOf,
                    'icon': module_info.icon,
                    'path': module_info.path
                });
            }
        });
        OS.modules.init = modules_merged;
        OS.modules.loaded = true;
        
        // If there is two programs, load the second
        // TODO: make a module to load custom apps at startup
        if(OS.modules.info.length==2 && OS.modules.info[1].id)
        {
            OS.startApp(OS.modules.info[1].id);
        }
    },

/**********************************************
 * Method: startApp
 * Start an application
 * 
 * Parameters:
 *     id - id of the application presents in OS.modules.init
 *     options - see Options
 *
 * Options:
 *      param - parameters
 *      onComplete - function to execute at the end
 * 
 * Returns:
 *     id - id
 **********************************************/    
    'startApp': function(id, options)
    {
        var id_unique = id;
        var interval = setInterval(function(){
            if(OS.modules.loaded===true){
                clearInterval(interval);
                var constructor = OS.modules.init[id];
                if($type(constructor)==='class'){
                    options = options || {};
                    var param = options.param || '';
                    var onComplete = options.onComplete || function(){};
                    var o = new OS.modules.init[id]();
                    if(o.multiple_instance){
                        id = id+'_'+$time();
                    }
                    else if($(id)){
                        OS.restoreWindow(id);
                        return true;
                    }
                    OS.modules.run[id] = o;
                    OS.modules.run[id].id = id;
                    OS.modules.run[id].start(param);
                    onComplete();
                    
                    // Update Adresse bar
                    OS.changeLocationState({'w':id,'a':id_unique});
                    
                    return true;
                }
                return false;
            }
            return false;
        }, 100);
        return false;
    },

/**********************************************
 * Variable: generic_module
 * Base module
 **********************************************/ 
    'generic_module': {
        'path': '',
        'id': '',
        'url': '',
        'icon': 'themes/default/images/icons/kblackbox16.png',
        'label': 'NoNameApp',
        'content': 'Application loading ...',
        'childOf': 'desktop',
        'width': false,
        'height': false,
        'minx': 400,
        'miny': 200,
        'resizable': true,
        'removeConfirm': false,
        'multiple_instance': false,
        'initialize': function() {
            // Auto adapt width and height if nothing specified
            if(!this.width){
                var width = OS.desktop.scrollWidth / 2;
                if(width>=this.minx){
                    this.width = width;
                }
                else{
                    this.width = this.minx;
                }
            }
            if(!this.height){
                var height = OS.desktop.scrollHeight / 2;
                if(height>=this.miny){
                    this.height = height;
                }
                else{
                    this.height = this.miny;
                }
            }
        }
    },

/**********************************************
 * Variable: container_module
 * Virtual module if the parent set in the module does not exist
 **********************************************/ 
    'container_module': {
        
        'childOf': 'desktop',
        'icon': 'themes/default/images/icons/package_programs.png',
        'start': function()
        {
            var id = this.id;
            OS.createWindow(
                this.label,
                this.id,
                {
                    'height': 100,
                    'width': 350,
                    'btnMaximize': false,
                    'onCreated': function()
                    {
                        OS.modules.info.each(function(mod)
                        {
                            if($type(mod)==='object' && mod.childOf===id)
                            {
                                OS.createIcon(id+'_'+mod.id, {
                                    'label': mod.label,
                                    'parent': id,
                                    'container': id+'_content',
                                    'icon': 'themes/default/images/icons/run.png',
                                    'cmd': ["OS.startApp('"+mod.id+"')", "function(){}"]
                                });
                            }
                        });
                    }
                }
            );
        }
    }
});//End of the os class

