/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
    Web 2.0 Operating System
*/
/*global $, $A, $random, $time, $type, Element, os, Fx */
/**********************************************
 * Class: OS.fx
 * Effects for Elements
 **********************************************/
OS.fx = {
/**********************************************
 * Method: move_in_nw
 * Move window in desktop from NW
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     index - index of the window element
 **********************************************/
    'move_in_nw': function(item, index)
    {
        item.style.left = '0px';
        item.style.top = '0px';
        var fx = new Fx.Morph(item,
        {
            'duration': 600,
            'transition': Fx.Transitions.Expo.easeOut,
            'onStart': function()
            {
                item.style.visibility = 'visible';
            }
        }).start({
            'left': OS.windows.info[index][2],
            'top': OS.windows.info[index][3]
        });
    },

/**********************************************
 * Method: move_out_nw
 * Move window out of desktop to NW
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     index - index of the window Element
 **********************************************/
    'move_out_nw': function(item, index)
    {
        var parent_id = OS.windows.parents[index];
        var fx = new Fx.Morph(item,
        {
            'duration': 250,
            'wait': false,
            'transition': Fx.Transitions.Expo.easeIn,
            'onComplete': function()
            {
                OS.desktop.removeChild(item);
            }
        }).start({
            'width': 0,
            'height': 0,
            'left': OS.windows.info[index][2]-OS.windows.info[index][0] +'px',
            'top': OS.windows.info[index][3]-OS.windows.info[index][1] +'px'
        });
    },

/**********************************************
 * Method: move_in_elem
 * Move a window out in an Element direction
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     elem - element to go to
 **********************************************/
    'move_in_elem': function(item, elem)
    {
        item.style.overflow = 'hidden';
        var item_size = item.getSize();
        var elem_pos = elem.getPosition();
        var elem_size = elem.getSize();
        var fx = new Fx.Morph(item,
        {
            'duration': 500,
            'wait': false,
            'transition': Fx.Transitions.Back.easeIn,
            'onComplete': function(){
                item.style.display = 'none';
            }
        }).start({
            'width': elem_size.x,
            'height': elem_size.y,
            'left': elem_pos.x +'px',
            'top': elem_pos.y +'px'
        });
    },

/**********************************************
 * Method: move_out_elem
 * Move a window from an Element direction out
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     index - index of the window Element
 *     elem - Element coming from
 **********************************************/
    'move_out_elem': function(item, index, elem)
    {
        var fx = new Fx.Morph(item,
        {
            'duration': 500,
            'wait': false,
            'transition': Fx.Transitions.Back.easeOut,
            'onStart': function(){
                item.style.display = '';
            }
        }).start({
            'width': OS.windows.info[index][0] +'px',
            'height': OS.windows.info[index][1] +'px',
            'left': OS.windows.info[index][2] +'px',
            'top': OS.windows.info[index][3] +'px'
        });
        item.style.overflow = 'auto';
    },

/**********************************************
 * Method: maximize_window
 * Maximize a window with ease
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     onMaximize - function to execute on fx complete
 **********************************************/
    'maximize_window': function(item, onMaximize)
    {
        var fx = new Fx.Morph(item,
        {
            'onComplete': onMaximize,
            'duration': 500,
            'wait': false,
            'transition': Fx.Transitions.Back.easeIn
        }).start({
            'width': OS.dw +'px',
            'height': OS.dh +'px',
            'left': -1,
            'top': 0
        });
    },

/**********************************************
 * Method: unmaximize_window
 * Unmaximize a window with ease
 * 
 * Parametres:
 *     item - window Element to apply the fx
 *     onUnMaximize - function to execute on fx complete
 **********************************************/
    'unmaximize_window': function(item, info, onUnMaximize)
    {
        var fx = new Fx.Morph(item,
        {
            'onComplete': onUnMaximize,
            'duration': 500,
            'wait': false,
            'transition': Fx.Transitions.Back.easeOut
        }).start({
            'width': info[0] +'px',
            'height': info[1] +'px',
            'left': info[2] +'px',
            'top': info[3] +'px'
        });
    }
};
