/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
    Web 2.0 Operating System
*/
/*global $, $A, $random, $time, $type, Class, Element, Options, console, os, Drag, $$, Fx, document */
/**********************************************
 * File: gui.js
 * 
 * Group: Examples
 * TODO
 **********************************************/
/**********************************************
 * Group: box
 * GUI Box
 **********************************************/
OS.gui = new Class({
    'childs': [],
    'elements': {
        'box': '',
        'container': '',
        'previous': '',
        'separator': '',
        'vbox_c': false
    },
    /**********************************************
     * Properties: options
     * Options for the object
     *
     * Example:
     *  var o1 = new OS.gui({'type': 'form'});
     *  var o2 = new OS.gui({
     *      'resizable': true,
     *      'orientation': 'v'
     *  });
     * 
     * Parameters:
     *  type - can be: div, form, fieldset or false.
     *         If set to boolean false, elements are directly inject into the window.
     *  label - used for type fieldset
     *  resizable - Default: false, resizable panel
     *  orientation - Default: 'h', can be: 'v', 'h'. It's the orientation of the panel
     *  positions - top, left, right, bottom
     *  sizes - min (default: 40), max (40), width, height
     *  className - CSS classes
     **********************************************/
    'options': {
        'type': 'div',
        'label': false,
        'resizable': false,
        'orientation': 'h',
        'className': '',
        'positions': {
            'top': false,
            'left': false,
            'right': false,
            'bottom': false
        },
        'sizes': {
            'min': 20,
            'max': 20,
            'width': false,
            'height': false
        }
    },
    
    /**********************************************
     * Method: initialize
     * Constructor of the object
     * 
     * Parameters:
     *     options - see <options>
     **********************************************/
    'Implements': Options,
    'initialize': function(options)
    {
        this.setOptions(options);
    },

    /**********************************************
     * Method: add
     * Add an Element to the gui object
     * 
     * Parameters:
     *     element - Element(s) to add
     **********************************************/
    'add': function()
    {
        var element = new Element('div', {'class':'gui'});
        $A(arguments).each(function(arg){
            if($type(arg)==='element')
            {
                arg.addClass('gui');
                arg.inject(element);
            }
        });
        this.childs.push(element);
    },
    
    /**********************************************
     * Method: inject
     * Inject gui elements in a window
     * TODO: Possibility to inject a box inside the element before it's in the DOM
     * 
     * Parameters:
     *     container - Container the box must go to. 
     *          Possible values: HTML Element, 
     *                           GUI Object Instance (to nest), 
     *                           ID of a window (it goes into the id_content), 
     *                           ID of HTML Element
     **********************************************/
    'inject': function(container)
    {
        if(container) {
            // Container is an Element
            if($type(container)==='element')
            {
                this.elements.container = container;
            }
            // Container is a gui object instance
            else if(container.elements && container.elements.box)
            {
                this.elements.container = container.elements.box;
            }
            // Container is a window ID
            else if($(container+'_content'))
            {
                this.elements.container = $(container+'_content');
            }
            // Container is an ID
            else if($(container))
            {
                this.elements.container = $(container);
            }
        }
        
        if(this.elements.container)
        {
            // Horizonzal box
            if(this.options.orientation==='h') {
                this.hbox();
            }
            // Vertical box
            else if(this.options.orientation==='v') {
                this.vbox();
            }
            
            // Inject Box
            if(this.elements.vbox_c) {
                this.elements.vbox_c.inject(this.elements.container);
                this.elements.previous = this.elements.vbox_c.getPrevious('.'+this.elements.container.id);
            }
            else {
                this.elements.box.inject(this.elements.container);
                this.elements.previous = this.elements.box.getPrevious('.'+this.elements.container.id);
            }
            // Inject Separator
            if(this.elements.previous) {
                this.elements.separator.inject(this.elements.box, 'before');
            }
            
            // Set initial positions
            this.setPositions();
        }
        return this.elements.container;
    },
    
    /**********************************************
     * Method: box
     * This is the principal window content container,
     * it defines a basic render to the content.
     * 
     * It takes the form of: Form, fieldset or
     * div container
     *
     * id is autogenerated from: current timestamp +
     * random number from 0 to 60 + string '_gui_box'
     *
     * Width and height, if specified, are stored
     * in the MT storage from here.
     * 
     * Parameters:
     *     childs - array of Elements to include in
     *              the container
     *     className - class name
     **********************************************/
    'box': function(childs, className)
    {
        var id = $time()+'_'+$random(0, 100)+'_gui_box';
        
        //FIXME: find a solution for the type
        // Div
        if(this.options.type==='div')
        {
            var container = new Element('div', {
                'id': id,
                'class': className+' '+this.options.className
            });
        }
        // Form
        else if(this.options.type==='form')
        {
            var container = new Element('form', {
                'id': id,
                'class': className+' '+this.options.className,
                'method': 'post',
                'action':'',
                'events': {
                    'submit': function(){ return false; }
                }
            });
        }
        // Fieldset
        else if(this.options.type==='fieldset')
        {
            var container = new Element('fieldset', {
                'id': id,
                'class': className+' '+this.options.className
            });
            var legend = new Element('legend', {'html': this.options.label});
            legend.inject(container);
        }
        
        //Childs
        if(childs.length>0) {
            childs.each(function(child)
            {
                if($type(child)==='element')
                {
                    child.inject(container);
                }
            });
        }
        return container;
    },
    
    /**********************************************
     * Method: hbox
     * Horizontal box wrapper
     **********************************************/
    'hbox': function()
    {
        this.elements.box = this.box(this.childs, 'hbox '+this.elements.container.id);
        this.elements.separator = this.separator(this.elements.box.id, this.elements.container.id);
    },
    
    /**********************************************
     * Method: vbox
     * Vertical box wrapper
     **********************************************/
    'vbox': function()
    {       
        // Check if there is a vbox container (vbox_c)
        var last = this.elements.container.getLast();
        var vbox_c = (last && last.hasClass('vbox_c')) ? last : false;
        if(vbox_c)
        {
            this.elements.box = this.box(this.childs, 'vbox '+vbox_c.id);
            this.elements.separator = this.separator(this.elements.box.id, vbox_c.id);
            this.elements.container = vbox_c;
        }
        // No Container, create an horizontal box container
        else
        {
            return this.vbox_c();
        }
        return false;
    },
    
    /**********************************************
     * Method: vbox_c
     * Horizontal box container vbox wrapper
     **********************************************/
    'vbox_c': function()
    {
        this.elements.box = this.box(this.childs, 'vbox');
        var box = this.box([this.elements.box], 'vbox_c hbox '+this.elements.container.id);
        this.elements.separator = this.separator(box.id, this.elements.container.id, this.options.resizable, 'h');
        this.elements.vbox_c = box;
        this.elements.box.addClass(box.id);
    },
    
    /**********************************************
     * Method: setTop
     **********************************************/
    'setTop': function(box, x)
    {
        box.style.top = x+'px';
    },
    
    /**********************************************
     * Method: setRight
     **********************************************/
    'setRight': function(box, x)
    {
        box.style.right = x+'px';
    },
    
    /**********************************************
     * Method: setBottom
     **********************************************/
    'setBottom': function(box, x)
    {
        box.style.bottom = x+'px';
    },
    
    /**********************************************
     * Method: setLeft
     **********************************************/
    'setLeft': function(box, x)
    {
        box.style.left = x+'px';
    },
    
    /**********************************************
     * Method: setHeight
     **********************************************/
    'setHeight': function(box, x)
    {
        box.style.height = x+'px';
    },
    
    /**********************************************
     * Method: setWidth
     **********************************************/
    'setWidth': function(box, x)
    {
        box.style.width = x+'px';
    },
    
    /**********************************************
     * Method: setPositions
     *
     * Parameters:
     *  pos - position in px to place the separator and left or top of box 
     **********************************************/
    'setPositions': function(pos)
    {
        //FIXME: Case of the first VBOX forgotten everywhere for separator ???
        var c_size;
        // Set sizes in percent for a start
        this.autoAdapt();
        
        // Position with the Mouse
        if(pos) {
            // Horizontal
            if(this.options.orientation==='h') {
                this.setTop(this.elements.box, pos);
                this.setTop(this.elements.separator, pos);
                this.setHeight(this.elements.previous, this.elements.previous.style.top.toInt()+pos);
            }
            // Vertical
            else {
                this.setLeft(this.elements.box, pos);
                //FIXME: Not finished
                this.elements.previous.style.width = this.elements.previous.style.left.toInt()+pos+'px';
                this.elements.separator.style.left = pos+'px';
            }
        }
        // Position with Options
        else {
            // Horizontal
            if(this.options.orientation==='h') {
                c_size = this.elements.container.offsetHeight;
                if(this.options.positions.top) {
                    this.elements.box.style.top = this.options.positions.top+'px';
                    this.elements.separator.style.top = this.options.positions.top+'px';
                    if(this.elements.previous) {
                        //FIXME USELESS ? Try to find a case where it bugs
                        this.elements.previous.style.bottom = c_size-this.options.positions.top+'px';
                    }
                }
                if(this.options.positions.bottom) {
                    this.elements.box.style.bottom = this.options.positions.bottom+'px';
                }
                if(this.options.sizes.height) {
                    this.elements.box.style.height = this.options.sizes.height+'px';
                }
            }
            // Vertical
            else {
                c_size = this.elements.container.offsetWidth;
                // FIXME: This is redundant, just for test now.......
                if(this.elements.vbox_c) {
                    if(this.options.positions.top) {
                        this.elements.vbox_c.style.top = this.options.positions.top+'px';
                    }
                    if(this.options.sizes.height) {
                        this.elements.vbox_c.style.height = this.options.sizes.height+'px';
                    }
                }
                if(this.options.positions.left) {
                    this.elements.box.style.left = this.options.positions.left+'px';
                    this.elements.separator.style.left = this.options.positions.left+'px';
                    if(this.elements.previous) {
                        //FIXME USELESS ? Try to find a case where it bugs
                        this.elements.previous.style.right = c_size-this.options.positions.left+'px';
                    }
                }
                if(this.options.positions.right) {
                    this.elements.box.style.right = this.options.positions.right+'px';
                }
                if(this.options.sizes.width) {
                    this.elements.box.style.width = this.options.sizes.width+'px';
                }
            }
        }
    },
    
    /**********************************************
     * Method: autoAdapt
     * Adapt height or width in % depending on the box number,
     * only same type boxes taken into account
     **********************************************/
    'autoAdapt': function()
    {
        if(this.elements.container) {
            var check = this.elements.container.getElements('div.'+this.elements.container.id);
            var check_nb = check.length;
            var adapt, bot, style, size, pos1, pos2, sep, prev;
            
            adapt = 100 / (check_nb);
            bot = check_nb-1;
            
            check.each(function(b, i)
            {
                pos1 = adapt*i+'%';
                pos2 = adapt*bot+'%';
                sep = b.getPrevious('span.'+b.id);
                
                // Horizontal
                if(b.hasClass('hbox') || b.hasClass('vbox_c')) {
                    style = {
                        'top': pos1,
                        'bottom': pos2
                    }
                    if(sep) {
                        sep.style.top = pos1;
                    }
                }
                // Vertical
                else if(b.hasClass('vbox')){
                    style = {
                        'left': pos1,
                        'right': pos2
                    }
                    if(sep) {
                        sep.style.left = pos1;
                    }
                }
                bot-=1;
                b.set('styles', style);
            });
        }
    },
    
    /**********************************************
     * Method: separator
     * Box horizontal or vertical separator
     *
     * Parameters:
     *  box_id - box_id
     *  container_id - container_id
     *  resizable - true or false. Make the box resizable.
     *              Default: gui.options.resizable
     *  orientation - h or v. Should be gui.options.orientation apart if it's the first vbox (vbox_c).
     *                Default: gui.options.orientation
     **********************************************/
    'separator': function(box_id, container_id, resizable, orientation)
    {
        //FIXME: Refactor
        orientation = orientation ? orientation : this.options.orientation;
        resizable = resizable ? resizable : this.options.resizable;
        var min_size = this.options.sizes.min;
        var max_size = this.options.sizes.max;
        var gui = this;
        var className = "";
        if(resizable) {
            className = orientation+'separator_resizable';
            var event = {'mousedown': function(e) {
                // Separator clicked
                e.preventDefault();
                var sep = e.target;
                var next = sep.getNext('.'+container_id);
                var prev = sep.getPrevious('.'+container_id);
                if(next && prev)
                {
                    var pos1, pos2, diff, min, max, pos, container_pos;
                    var container = $(container_id);
                    var win = sep.getParents('div.window')[0];
                    var sep_ghost = sep.clone().inject(sep, 'before');
                    sep_ghost.addClass('separator_ghost');
                 
                    // Set minimum and maximum
                    if(orientation==='h')
                    {
                        diff = win.getPosition().y + container.getPosition(win).y;
                        min = prev.offsetTop + min_size;
                        max = next.offsetTop + next.offsetHeight - max_size;
                    }
                    else if(orientation==='v')
                    {
                        diff = win.getPosition().x + container.getPosition(win).x;
                        min = prev.offsetLeft + min_size;
                        max = next.offsetLeft + next.offsetWidth - max_size;
                    }
                    
                    // Event on separator move
                    container.addEvent('mousemove', function(e)
                    {
                        e.preventDefault();
                        // Get box height or width and mouse position
                        if(orientation==='h') {
                            pos = e.client.y - diff;
                        }
                        else if(orientation==='v') {
                            pos = e.client.x - diff;
                        }
                        // If position is outside min or max, set it to it :)
                        if(pos>max) {
                            pos = max;
                        }
                        else if(pos<min) {
                            pos = min;
                        }
                        
                        // Move the ghost
                        if(orientation==='h') {
                            sep_ghost.style.top = pos+'px';
                        }
                        else if(orientation==='v') {
                            sep_ghost.style.left = pos+'px';
                        }
                    });
                    // Set boxes positions when mouse released
                    container.addEvent('mouseup', function()
                    {
                        container.removeEvents('mousemove');
                        container.removeEvents('mouseup');
                        sep_ghost.destroy();
                        gui.setPositions(pos);
                    });
                }
            }};
        }
        else {
            className = orientation+'separator';
        }
        return new Element('span', {
            'id': orientation+'separator_'+box_id,
            'class': className+' '+box_id,
            'events': event
        });
    },
    
    /**********************************************
     * Group: elments
     * GUI Elements
     **********************************************/
    /**********************************************
     * Method: table
     * Table
     * 
     * Parameters:
     *     id - id
     *     options - cols, sum, data
     *
     * Example:
     * (start code)
     *  var gui = new this.gui();
     *  gui.table(id, {
     *      sum: [
     *          'col1',
     *          'col2'
     *      ],
     *      data:[
     *          'cell1',
     *          'cell2',
     *          'cell3',
     *          'cell1'
     *      ],
     *      style: 'width: 100%; height: 100%;'
     *  });
     *  gui.inject(id);
     * (end)
     **********************************************/
    'table': function(id, options)
    {
        var sum = options.sum ? options.sum : false;
        var cols = options.cols ? options.cols : sum ? sum.length : 3;
        var data = options.data ? options.data : [];
        var rows = Math.ceil(data.length / cols);
        var style = options.style ? options.style : '';
        var html = '';
        
        //Head
        if(sum && sum.length===cols)
        {
            html += '<thead><tr>';
            for(var i=0; i<cols; i+=1)
            {
                html += '<th>'+sum[i]+'</th>';
            }
            html += '</tr></thead>';
        }
        
        //Fill cells
        var col = 1;
        html += '<tbody>';
        data.each(function(cell, i)
        {
            if(col===1)
            {
                html += '<tr>';
            }
            html += '<td>'+cell+'</td>';
            if(col===cols && i!==(data.length -1))
            {
                html += '</tr>';
                col=0;
            }
            col+=1;
        });
        
        //Fill empty cells
        if(data.length % cols!==0)
        {
            var remains = (cols * rows) - data.length;
            for(var j=0; j<remains; j+=1)
            {
                html += '<td></td>';
            }
        }
        html += '</tr></tbody>';
        
        //Create Element
        var element = new Element('table',
        {
            'id': id+'_gui_table',
            'style': style,
            'html': html
        });
        
        this.add(element);
        return element;
    },

    /**********************************************
     * Method: pre
     * Preview, HTML balise <pre/>
     *
     * Parameters:
     *     id - id
     *     value - value inside pre
     *     options - no options
     **********************************************/
    'pre': function(id, options)
    {
        options = options ? options : {};
        var value = options.value ? options.value : '';
        var element = new Element('pre',
        {
            'id': id+'_gui_pre',
            'html': value
        });
        this.add(element);
        options.class ? element.addClass(options.class) : '';
        return element;
    },

    /**********************************************
     * Method: label
     * Element texte
     * 
     * Parameters:
     *     id - Element ID
     *     label - text contained in the Element
     *     options - See Options
     *
     * Options:
     *      className - Class to add to the Element
     *      forElem- id of the Element the label is attached to
     **********************************************/
    'label': function(id, label, options)
    {
        options = options ? options : {};
        var className = options.className ? options.className : '';
        var forElem = options.forElem ? options.forElem : '';
        var element = new Element('label',
        {
            'id': id+'_gui_label',
            'html': label,
            'for': forElem
        });
        this.add(element);
        return element
    },

    /**********************************************
     * Method: div
     * DIV Element
     * 
     * Parameters:
     *     id - Element ID
     *     options - See Options
     *
     * Options:
     *      html - Optional content of the DIV
     *      className - Class to add to the Element
     **********************************************/
    'div': function(id, options)
    {
        options = options ? options : {};
        var html = options.html ? options.html : '';
        var className = options.className ? options.className : '';
        var element = new Element('div',
        {
            'id': id+'_gui_div',
            'class': className,
            'html': html
        });
        this.add(element);
        return element
    },

    /**********************************************
     * Method: button
     * Element bouton
     * 
     * Parameters:
     *     id - Element ID
     *     label - texte contenu dans l'element
     *     options - action
     **********************************************/
    'button': function(id, label, options)
    {
        options = options ? options : {};
        var action = options.action ? options.action : function(){ return false; };
        var style = style ? style : '';
        var element = new Element('button',
        {
            'id': id+'_gui_button',
            'html': label,
            'style': style,
            'events': {'click': action}
        });
        this.add(element);
        options.class ? element.addClass(options.class) : '';
        return element;
    },
    
    /**********************************************
     * Method: reset
     * 
     * Parameters:
     *     id - id
     *     label - label
     *     options - return
     **********************************************/
    'reset': function(id, label, options)
    {
        options = options ? options : {};
        var element = new Element('input',
        {
            'id': id+'_gui_reset',
            'type': 'reset',
            'value': label
        });
        this.add(element);
        return element;
    },

    /**********************************************
     * Method: checkbox
     * Checkbox HTML Element
     * 
     * Parameters:
     *     id - id
     *     name - text in the element
     *     options - action, checked, name_before, class
     *
     * Returns:
     *     The checkbox
     **********************************************/
    'checkbox': function(id, name, options)
    {
        options = options ? options : {};
        var action = options.action ? options.action : function(){};
        var checked = options.checked ? 'checked' : '';
        var name_before = options.name_before ? options.name_before : false;
        var input = new Element('input',
        {
            'id': id+'_gui_checkbox',
            'type': 'checkbox',
            'class': 'checkbox',
            'checked': checked,
            'events': {'click': action}
        });
        var label = new Element('label',
        {
            'for': id+'_gui_checkbox',
            'html': name
        });
        if(name_before) {
            this.add(label, input);
        }
        else {
            this.add(input, label);
        }
        options.class ? input.addClass(options.class) : '';
        return input;
    },

    /**********************************************
     * Method: radio
     * Radio button element. All related radio
     * buttons must have the same ID's !
     * MUST BE USED IN A FORM ATM !
     * 
     * Parameters:
     *     name - element identifier, must be the same
     *          as all other radio buttons of the choice
     *          see <gui>
     *     value - value of the button
     *     label_html - Label text
     *     options - checked, action, name_before
     **********************************************/
    'radio': function(name, value, label_html, options)
    {
        options = options ? options : {};
        var id = name+'_'+$time()+'_'+$random(0,50)+'_gui_radio';
        var checked = options.checked ? 'checked' : '';
        var action = options.action ? options.action : function(){};
        var name_before = options.name_before ? options.name_before : false;
        var input = new Element('input',
        {
            'id': id,
            'name': name+'_gui_radio',
            'value': value,
            'type': 'radio',
            'checked': checked,
            'events': {'click': action}
        });
        var label = new Element('label',
        {
            'for': id,
            'html': label_html
        });
        if(name_before) {
            this.add(label, input);
        }
        else {
            this.add(input, label);
        }
        return input;
    },

    /**********************************************
     * Method: radioGet
     * Return the value of a group of radio button
     * (Independent from the instance of OS.gui)
     * 
     * Parameters:
     *     name - radio buttons' name
     *     childOf - parent id of the form
     **********************************************/
    'radioGet': function(name, childOf)
    {
        var value = false;
        var list = $(childOf).getChildren('form')[0][name+'_gui_radio'];
        $A(list).each(function(radio){
            if(radio.checked=='true' || radio.checked)
            {
                value = radio.get('value');
            }
        });
        return value;
    },

    /**********************************************
     * Method: radioSet
     * 
     * Parameters:
     *     value - value
     *     name - radio buttons' name
     *     childOf - parent id of the form
     **********************************************/
    'radioSet': function(value, name, childOf)
    {
        var list = $(childOf).getChildren('form')[0][name+'_gui_radio'];
        $A(list).each(function(radio){
            if(radio.value==value)
            {
                radio.set('checked', 'true');
            }
            else
            {
                radio.set('checked', 'false');
            }
        });
    },

    /**********************************************
     * Method: field
     * Element champs
     * 
     * Parameters:
     *     id - Element ID
     *     options - See Options
     * Options:
     *      value - Initial value of the field
     *      keydown - Action when any key is down
     *      focus - Action on focus, Default: reset field content if
     *              any default value is inside (not user content)
     *      style - Row style to apply
     *      label - Label of the field (click will focus on the field)
     *      size - Apparent size of the field, Default, maxlength or 20
     *      maxlength - Character limitation in the field, Default: 255
     *      className - class(es) to apply to the field
     **********************************************/
    'field': function(id, options)
    {
        options = options ? options : {};
        var style = options.style ? options.style : '';
        var value = options.value ? options.value : '';
        var className = options.className ? options.className : '';
        var maxlength = options.maxlength ? options.maxlength: 255;
        var size = options.size ? options.size: options.maxlength ? options.maxlength: 20;
        var keydown = options.keydown ? options.keydown : function(){};
        var focus = options.focus ? options.focus : function(e) {
            if(value===e.target.value) {
                e.target.value = '';
            }
        };
        var field = new Element('input',
        {
            'id': id+'_gui_field',
            'type': 'text',
            'class': className,
            'style': style,
            'value': value,
            'size': size,
            'maxlength': maxlength,
            'events': {'keydown': keydown, 'focus':focus}
        });
        if(options.label)
        {
            var label = new Element('label',
            {
                'for': id+'_gui_field',
                'html': options.label
            });
            this.add(label, field);
        }
        else
        {
            this.add(field);
        }
        return field;
    },

    /**********************************************
     * Method: textarea
     * Element champs multilignes
     * 
     * Parameters:
     *     id - Element ID
     *     options - value, keydown, focus, style, readonly, class
     **********************************************/
    'textarea': function(id, options)
    {
        options = options ? options : {};
        var value = options.value ? options.value : '';
        var style = options.style ? options.style : '';
        var readonly = options.readonly ? 'readonly' : false;
        var keydown = options.keydown ? options.keydown : function(){};
        var focus = options.focus ? options.focus : function(e) {
            if(value===e.target.value)
            {
                e.target.value = '';
            }
        };
        var label = options.label ? new Element('label',
        {
            'for': id+'_gui_textarea',
            'html': options.label
        }) : new Element('div');
        var text = new Element('textarea',
        {
            'id': id+'_gui_textarea',
            'html': value,
            'style': style,
            'readonly': readonly,
            'events': {'keydown': keydown, 'focus':focus}
        });
        this.add(label, text);
        options.class ? text.addClass(options.class) : '';
        return text;
    },

    /**********************************************
     * Method: option
     * Element de selecton d'options
     * 
     * Parameters:
     *     id - Element ID
     *     options - value, change, label, options, multiple, value, selected
     **********************************************/
    'option': function(id, options)
    {
        options = options ? options : {};
        var value = options.value ? options.value : '';
        var multiple = options.multiple ? options.multiple : false;
        var selected = options.selected ? options.selected : false;
        var change = options.change ? options.change : function(){};
        var label = options.label ? new Element('label',
        {
            'for': id+'_gui_option',
            'html': options.label
        }) : new Element('div');
        var select = new Element('select',
        {
            'id': id+'_gui_option',
            'multiple': multiple,
            'events': {'change':change}
        });
        var elem;
        options.options.each(function(option)
        {
            if(selected && option[1]===selected)
            {
                elem = new Element('option',
                {
                    'html': option[0],
                    'value': option[1],
                    'selected': 'selected'
                }).inject(select);
            }
            else
            {
                elem = new Element('option',
                {
                    'html': option[0],
                    'value': option[1]
                }).inject(select);
            }
        });
        this.add(label, select);
        return select;
    }
    
    //TODO: More more more
    //menubar:'',
    //statusbar:'',
    //list:'',
    //spin:'',
    //sliders:'',
});


