/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

             v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.icon
 * All Icons methods (not separated from OS.core->TODO)
 **********************************************/
OS.core.implement({
/**********************************************
 * Method: createIcon
 * Icon creation
 *
 * Parametres:
 *     name - Icon id
 *     options - label, parent, container, icon, cmd, posx, posy, minx, maxx, miny, maxy
 **********************************************/
    'createIcon': function(name, options)
    {
        var id = 'icon_'+name.replace('/ /', '-');
        var label_max = 6;
        var trunc_char = '~';
        
        if(!$(id))
        {
            options = options ? options : {};
            var label     = options.label ? options.label : name;
            var container = options.container ? options.container : 'desktop';
            var parent    = options.parent ? options.parent : 'desktop';
            var image     = options.icon ? options.icon : '/default/images/icons/unknown.png';
            var actions   = options.cmd ? options.cmd : [function(){}, function(){}];
            var posx      = options.x ? options.x : 0;
            var posy      = options.y ? options.y : 0;
            var re_res = /(.*)\.(.{3,4})/.exec(label);
            
            if(re_res)
            {
                var ext = re_res[2];
                label = re_res[1].substr(0, (label_max - ext.length - 2)) + trunc_char + '.'+ext;
            }
            else if(label.length > label_max)
            {
                label = label.substr(0, label_max - 1) + trunc_char;
            }
            
            var containerObj = (parent==='desktop') ? $('desktop') : $(container);
            var action_info = /^os\.modules\.(\w*\.\w*)[\[.*\]|\(.*\)].*$/.exec(actions[0]);
            if($type(action_info)==='array') {action_info=action_info[1];}
            else {action_info = 'unknown action';}
            
            var image_container = new Element('div',
            {
                'class': 'icon_image',
                'style': 'background-image: url("'+image+'");'
            });
            var text_container = new Element('div',
            {
                'class': 'icon_text',
                'html': label
            });
            var icon = new Element('div',
            {
                'id': id,
                'class': 'icon',
                'style': 'visibility: hidden',
                'events': {
                    'click':function()
                    {
                        var last_icon = $(OS.selected_icon);
                        if(last_icon)
                        {
                            var children = last_icon.getChildren();
                            last_icon.removeClass('icon_selected');
                            children[0].removeClass('icon_image_selected');
                            children[1].removeClass('icon_text_selected');
                        }
                        OS.selected_icon=id;
                        icon.addClass('icon_selected');
                        image_container.addClass('icon_image_selected');
                        text_container.addClass('icon_text_selected');
                    },
                    'mouseenter': function()
                    {
                        icon.addClass('icon_hover');
                        image_container.addClass('icon_image_hover');
                        text_container.addClass('icon_text_hover');
                    },
                    'mouseleave': function()
                    {
                        icon.removeClass('icon_hover');
                        image_container.removeClass('icon_image_hover');
                        text_container.removeClass('icon_text_hover');
                    },
                    'dblclick': function(e){
                        e.preventDefault();
                        eval(actions[0]);
                    },
                    'contextmenu': function(e){
                        e.preventDefault();
                        eval(actions[1]);
                    }
                }
            });
            
            image_container.inject(icon);
            text_container.inject(icon);
            icon.inject(containerObj);
            
            // Bouger les icones
            var drag = new Drag.Move(id,
            {
                'container': container,
                'onStart': function(item)
                {
                    item.style.zIndex = 99999;
                },
                'onComplete': function(item)
                {
                    item.style.zIndex = '';
                    OS.iconPosition(id, item);
                }
            });
            
            // Positionnement des icones
            var item = $(id);
            if(posx && posy)
            {
                item.style.left = posx+'px';
                item.style.top = posy+'px';
            }
            else
            {
                posx=0;
                posy=0;
                item.style.left = '0px';
                item.style.top = '0px';
            }
            this.icons.info.push([50, 57, posx, posy]);
            this.icons.xref.push(id);
            this.icons.parents.push(parent);
            this.icons.container.push(container);
            
            var indexPar  = this.windows.xref.indexOf(parent);
            var desktop = this.desktop;
            var pos;
            
            // Waiting for the window to be in place (the animation to finish)
            var interval = setInterval(function()
            {
                OS.showLoader($(container));
                pos = $(parent).getPosition(desktop);
                if(pos.x===OS.windows.info[indexPar][2] && pos.y===OS.windows.info[indexPar][3])
                {
                    // Position the icons
                    OS.hideLoader($(container));
                    clearInterval(interval);
                    OS.iconPosition(id, item);
                    item.style.visibility = 'visible';
                }
            }, 5);
        }
    },

/**********************************************
 * Method: cleanIcons
 * Remove all icons from a window
 *
 * Parametres:
 *     id - id of the window
 *     item - Element of the window (optional)
 **********************************************/
    'cleanIcons': function(id, item)
    {
        if(!item)
        {
            item = $(id);
        }
        var icon_id, parent;
        OS.icons.info.each(function(icon, index)
        {
            parent = OS.icons.parents[index];
            if(parent==id)
            {
                $(OS.icons.xref[index]).dispose();
                OS.icons.info = OS.unset(OS.icons.info, index);
                OS.icons.xref = OS.unset(OS.icons.xref, index);
                OS.icons.parents = OS.unset(OS.icons.parents, index);
                OS.icons.container = OS.unset(OS.icons.container, index);
            }
        });
    },

/**********************************************
 * Method: iconPosition
 * Place the icon to the nearest location according to the given position
 *
 * Parametres:
 *     id - id of the icon
 *     item - Element of the icon (optional)
 *     cycle - recursive try limit (developpement only ?)
 **********************************************/
    'iconPosition': function(id, item, cycle)
    {
        if(!item)
        {
            item = $(id);
        }
        var padding = item.getStyle('padding').toInt();
        var info = this.iconInformation(id, item);
        var w = info[0]; var h = info[1];
        var x = info[2]; var y = info[3];
        var c1 = [x,y,(x+w),(y+h)];
        var index = this.icons.xref.indexOf(id);
        var parentId1 = this.icons.parents[index];
        var indexParent = this.windows.xref.indexOf(parentId1);
        var container_id = this.icons.container[index];
        var container_w = $(container_id).getSize().x;
        var check = 0;
        cycle = cycle ? cycle : 0;
        this.icons.info.each(function(i,j){
            var idi = OS.icons.xref[j];
            var parentId2 = OS.icons.parents[j];
            var wi = i[0]; var hi = i[1];
            var xi = i[2]; var yi = i[3];
            var c2 = [xi, yi, (xi+wi), (yi+hi)];
            if(OS.squareIntersect(c1,c2) && idi!==id && parentId1===parentId2) {
                var posxmax = x+(2.5*w);
                if(posxmax<container_w) {
                    info[2] = xi+wi+padding;
                    info[3] = yi;
                }
                else {
                    info[2] = 0;
                    info[3] = yi+hi+padding;
                }
                item.style.left = info[2]+'px';
                item.style.top = info[3]+'px';
                check +=1;
            }
        });
        this.icons.info[index]=info;
        if(check===0 || cycle > 50) {
            cycle > 50 ? console.log('Core: iconPosition: Too much recursions') : '';
            return true;
        }
        else {
            cycle += 1;
            this.iconPosition(id, item, cycle);
        }
        return true;
    },

/**********************************************
 * Method: iconInformation
 * Icons Information (positions and dimensions)
 * Detecter les dimensions et positions des icones
 **********************************************/
    'iconInformation': function(id, item)
    {
        if(id)
        {
            if(!item)
            {
                item = $(id);
            }
            var parent_titleH;
            var index = this.icons.xref.indexOf(id);
            var parent_id = this.icons.parents[index];
            var container_id = this.icons.container[index];
            if(parent_id!=='desktop') {parent_titleH = $(parent_id+'_title').offsetHeight;}
            else {parent_titleH = 0;}
            var parent_index = this.windows.xref.indexOf(parent_id);
            var parent_info = this.windows.info[parent_index];
            var container_info = $(container_id).getCoordinates();
            var coord = item.getCoordinates();
            var padding = item.getStyle('padding').toInt();
            return [
                (coord.width + padding),
                 coord.height,
                (coord.left - container_info.left),
                (coord.top - container_info.top)];
        }
        return false;
    }
});

