/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
    Web 2.0 Operating System
*/
/*global Browser, alert, document, navigator, session_id, source, window*/
/**********************************************
 * Class: init
 * Initialisation methods, without mootools.
 * Creates the OS namespace.
 **********************************************/
var OS={
    'name': 'takos.info',
    /**********************************************
     * Method: source
     * Tends to be a cross-browser javascript loader
     **********************************************/
    'source': function(src, next_func)
    {
        next_func = next_func ? next_func : function(){};
        var script = document.createElement('script');
        script.type = "text/javascript";
        //script.src = 'Kernel/Server/syscall.php?session_id='+session_id+'&req=includeJs&file='+src;
        script.src = src;
        document.getElementsByTagName('head').item(0).appendChild(script);
        
        // Browser detection
        // FIXME: Opera is not compatible ?
        if(Browser===false)
        {
            Browser = { 'Engine': {
                'trident': function(){
                    if(document.all) { return true; }
                    else { return false; }
                },
                'webkit': function(){
                    if(navigator.vendor==='KDE' || ( document.childNodes && !document.all && !navigator.taintEnabled )) { return true; }
                    else { return false; }
                },
                'gecko': function(){
                    if(navigator.product==='Gecko' && !navigator.savePreferences) { return true; }
                    else { return false; }
                }
            }};
        }
        
        /* for Safari, Konqueror and Chrome (Webkit)*/
        if(Browser.Engine.webkit)
        {
            script.addEventListener('load', next_func, false);
        }
        /* for Mozilla/Gecko */
        else if(Browser.Engine.gecko)
        {
            script.onload = next_func;
        }
        /* for Internet Explorer */
        else if(Browser.Engine.trident)
        {
            var script_ie = document.createElement('script');
            script_ie.type = "text/javascript";
            script_ie.id = "__ie_onload";
            script_ie.defer = "defer";
            //script_ie.src = 'javascript:void(0)';
            document.getElementsByTagName('head').item(0).appendChild(script_ie);
            var script_ie_elem = document.getElementById("__ie_onload");
                    
            script_ie_elem.onreadystatechange = function() {
                if (this.readyState==="complete") {
                    next_func();
                }
            };
        }
        else
        {
            alert('Your browser may  not be compatible !');
            script.onload = next_func;
        }
    },
    

    /**********************************************
     * Method: sourceR
     * Iteratively or recursivly load modules files JS
     *
     * Parameters:
     *    files - Files array
     *    pos - key of the element to load in the array
     *    fn - function to execute when finished
     *    parallel - load the files parallelly. Defaut: false
     **********************************************/
    'sourceR': function(files, pos, fn, parallel)
    {
        var len = files.length;
        pos = pos || 0;
        fn = fn || function(){};
        parallel = parallel || false;
        OS.load.update(files[pos]);
        if(parallel){
            for(var i=0; i<len; i++){
                OS.source(files[i], function(){});
            }
        }
        else{
            OS.source(files[pos], function(){
                pos += 1;
                if(pos<len) {
                    OS.sourceR(files, pos, fn);
                }
                else {
                    fn();
                }
            }); 
        }
    },
    
    /**********************************************
     * Group: connection
     * Test the average connection speed
     *
     * Properties:
     *   file_src - path to generated file
     *   file_size - file size in Bytes
     *   time1 - beginning of test in unix time
     *   time2 - end of test in unix time
     *   speed - speed result in KBytes per seconds
     **********************************************/
    'connection': {
        'file_src': 'Kernel/Server/syscall.php?req=connectionTest&session_id='+session_id,
        'file_size': 307200,
        'file_id': new Date().getTime()+'_image',
        'time1': 0,
        'time2': 0,
        'speed': 0,
        /**********************************************
         * Method: time
         * Return current timestamp
         **********************************************/
        'time': function(){
            var t = new Date();
            return t.getTime();
        },
        /**********************************************
         * Method: finish
         * Compute speed and return it to systemInfo
         * Fired when iframe content is loaded
         **********************************************/
        'finish': function(){
            OS.connection.time2 = OS.connection.time();
            var time = OS.connection.time2 - OS.connection.time1;
            var speed = Math.round((OS.connection.file_size/1024)/(time/1000));
            var file = document.getElementById(OS.connection.file_id);
            document.getElementsByTagName('body').item(0).removeChild(file);
            OS.connection.speed = speed;
            OS.systemInfo('Connection : '+speed+' KB/s environ in '+time+' ms');
        },
        /**********************************************
         * Method: test
         * Start the timer and load the file.
         * Function to start to test the speed.
         * Get the result in OS.connection.speed (in KBytes)
         **********************************************/
        'test': function(){
            OS.connection.time1 = OS.connection.time();
            var file = document.createElement('iframe');
            file.id = OS.connection.file_id;
            file.src = OS.connection.file_src+'&size='+OS.connection.file_size+'&'+OS.connection.time1;
            file.onload = OS.connection.finish;
            file.style.visibility = 'hidden';
            document.getElementsByTagName('body').item(0).appendChild(file);
        }
    },
    
    /**********************************************
     * Group: load
     * Load javascript libraries one by one while
     * displaing a basic loading status
     *
     * Example:
     *  (start code)
     *  $('menu_load').addEvent('click', function(){
     *      OS.load.start(['1.js','2.js','3.js'], 'Titre', 'test1.js');
     *      OS.load.update('test2.js');
     *      OS.load.update('test3.js');
     *      OS.load.finish.delay(2000);
     *  });
     *  (end)
     **********************************************/
    'load': {
        /**********************************************
         * Method: start
         * Display loading status and set it to 0%
         **********************************************/
        'start': function(src, title, first)
        {
            src = src ? src : [];
            Browser = false;
            document.getElementById('os').style.display = 'none';
            var loading = document.createElement('div');
            loading.id = 'loading';
            loading.innerHTML = '<div id="loading_inner"><h1>'+title+'</h1><div id="loading_pc">0%</div><div id="loading_lib">'+first+'<br/></div></div>';
            document.getElementsByTagName('body').item(0).appendChild(loading);
            OS.load.int = 100 / src.length;
        },
        /**********************************************
         * Method: update
         * Update the status with the lib name and %
         **********************************************/
        'update': function(name)
        {
            var pc = document.getElementById('loading_pc'); 
            var lib = document.getElementById('loading_lib');
            if(pc && lib)
            {
                pc.innerHTML =  parseInt(pc.innerHTML, 10)+ parseInt(OS.load.int, 10) + '%';
                lib.innerHTML += name+'<br />';
            }
        },
        /**********************************************
         * Method: finish
         * Remove the loading status of the screen
         **********************************************/
        'finish': function()
        {
            document.getElementById('os').style.display = '';
            var elem = document.getElementById('loading');
            document.getElementsByTagName('body').item(0).removeChild(elem);
        }
    },

    /**********************************************
     * Method: jsonDb
     * Match cols with rows in compressed JSON
     * 
     * Parametres:
     *  obj - object
     *
     * Returns:
     *  array - array
     **********************************************/
    'jsonDb': function(obj)
    {
        var out = [];
        if(obj)
        {
            var i, j, row, col;
            for(i in obj['rows'])
            {
                if(parseInt(i, 10)>=0)
                {
                    row = [];
                    for(j in obj['rows'][i])
                    {
                        if(parseInt(j, 10)>=0)
                        {
                            col = obj['cols'][j];
                            row[col] = obj['rows'][i][j];
                        }
                    }
                    out.push(row);
                }
            }
        }
        return out;
    },
    
    /**********************************************
     * Group: ext
     * Exterior libraries
     **********************************************/
    'ext': {},
    
    /**********************************************
     * Group: modules
     * Information about modules
     **********************************************/
    'modules': {
        /**********************************************
         * Property: list
         * List of declared modules 
         **********************************************/
        'list':[],
        /**********************************************
         * Property: info
         * Display information about usable modules
         * (Declared and virtual modules)
         **********************************************/
        'info':[],
        /**********************************************
         * Property: init
         * Classes of declared modules
         **********************************************/
        'init':[],
        /**********************************************
         * Property: run
         * Runtime modules instances
         **********************************************/
        'run':{},
        /**********************************************
         * Property: loaded
         * loading state
         **********************************************/
        'loaded': false
    },
    /**********************************************
     * Property: start
     * Load base modules and run the kernel
     **********************************************/
    'start': function()
    {
        // Core files, order is super dupper important
        var core = [
            'Kernel/Ext/mootools-1.2-core.js',
            'Kernel/Ext/mootools-1.2-more.js',
            //'Kernel/Ext/mt-overload.js',
            //'Kernel/Ext/mt-idleupdate.js',
            
            'Kernel/OS/Core.js',
            'Kernel/OS/Context.js',
            'Kernel/OS/Window.js',
            'Kernel/OS/Icon.js',
            'Kernel/OS/Fx.js',
            
            'Kernel/OS/Gui.js',
            'Kernel/OS/Box.js',
            'Kernel/OS/Box/Elements.js',
            'Kernel/OS/Box/Panel.js',
            
            'Kernel/OS/Widget.js',
            'Kernel/Widgets/Menu.js',
            'Kernel/Widgets/Taskbar.js'
        ];
        
        //Initial
        OS.load.start(core, this.name, 'Kernel/OS/Init.js');
        OS.sourceR(core, 0, function(){
            // Run Kernel
            OS.run();
        });
    },
    /**********************************************
     * Property: run
     * Instanciantion of kernel
     **********************************************/
    'run': function()
    {
        var o = this;
        window.addEvent('domready', function(){
            OS.core.implement(o);
            OS = new OS.core();
            OS.load.finish();
        });
    }
};

/**********************************************
 * Section: START
 **********************************************/
OS.start();
