/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
        Web Operating System
*/
/**********************************************
 * Group: OS.widget
 * Widget environnement arrays and abstract class
 **********************************************/
OS.widget = {
/**********************************************
 * Properties: run
 * Copie of the objects instances
 **********************************************/
'run': [],
/**********************************************
 * Properties: OS.widget.init
 * Classes of the widget
 **********************************************/
'init': {},
/**********************************************
 * Class: OS.widget.base
 * Abstract class to herit by widgets.
 **********************************************/
'base': new Class({
    /**********************************************
     * Properties: Properties
     **********************************************/
    'Implements': Options,
    'id': 'base_widget',
    'options': {
        'movable': false,
        'name': 'noname',
        'container': 'os'
    },
    'element': false,
    'drag': false,
    
    /**********************************************
     * Method: initialize
     **********************************************/
    'initialize': function(options){
        // Merge passed options with default
        this.setOptions(options);
        // Add the widget to the container
        if($(this.options.container)){
            this.create();
            this.add();
            this.contextmenu();
        }
        if(this.options.movable) {
            this.move();
        }
    },
    
    /**********************************************
     * Method: create
     **********************************************/
    'create': function(){
        if($(this.id)){
            this.id = this.id+'_'+$time();
        }
        this.element = new Element('div',{
            'id': this.id,
            'class': 'widget',
            'html': 'Hello world'   
        });
    },
    
    /**********************************************
     * Method: add
     **********************************************/
    'add': function(){
        this.element.inject(this.options.container);
        OS.widget.run[this.id] = this;
    },
    
    /**********************************************
     * Method: remove
     **********************************************/
    'remove': function(){
        this.element.dispose();
        OS.widget.run = OS.unset(OS.widget.run, this.id);
    },
    
    /**********************************************
     * Method: move
     * If options.movable is True, the widget is
     * movable inside the container.
     **********************************************/
    'move': function(){
        this.drag = new Drag.Move(this.element, {
            'container': this.options.container
        });
    },
    
    /**********************************************
     * Method: contextmenu
     **********************************************/
    'contextmenu': function(){
        var container = this.element;
        var o = this;
        var menu = new OS.context({
            'container': container,
            'list': [{
                'name': 'Remove widget',
                'icon': 'themes/default/images/icons/cancel.png',
                'action': function(){
                    OS.infoWindow('Remove widget', 'remove_widget_question', {
                        'content': 'Are you sure you want to remove <b>'+o.options.name+'</b> ?',
                        'actions': {
                            'No': function(){},
                            'Yes': function(){o.remove();}
                        },
                        'modal': false
                    });
                }
            }]
        });
    }
})};
