/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.window
 * All Windows methods (not separated from OS.core->TODO)
 **********************************************/
OS.core.implement({
/**********************************************
 * Method: createWindow
 * Creates a new window FIXME: Windows can only be childOf an other window or desktop! could be any Element as a widget
 *
 * Parameters:
 *          label - Window's title
 *          id
 *          option - see Options
 * 
 * Options:
 *   content - String or HTML Element that goes to window_content
 *   posx - Horizontal position of the win, Default: 0 from parent
 *   posy - Vertical position of the win, Default: 0 from parent
 *   width - width
 *   height - height
 *   minx - minimum width
 *   miny - minimum height
 *   maxx - maximum width
 *   maxy - maximum height
 *   url - request HTML content (DEPRECIATED)
 *   onComplete - if url, when reuquest is complete (DEPRECIATED)
 *   postData - data for the request (DEPRECIATED)
 *   multiple_instance - The window may have numbers of instances (an id is generated for each one in run)
 *   removeConfirm - Ask before closing the window
 *   onClose - functino to execute when the window is being closed
 *   childOf - parent id
 *   onResizeEnd - function to execute when resize is finished
 *   onResize - function to execute at EVERY mouse move during resize (Caution, this may be slow with to big function!)
 *   onMaximize - function to execute when window is maximized
 *   onUnMaximize - function to execute when window is unmaximized
 *   onCreated - function to execute after the window has been created
 *   resizable - Window is resizable, Default: false
 *   closeChild - If any child, ask to close all windows associated
 *   modal - Default: false, id of the element to block when this window appears
 *   statusbar - Status bar
 *   className - class for the main window Element
 *  
 * Returns:
 *     id - id of the created window (can be different from input in case of multi-instance application)
 **********************************************/
    'createWindow': function(label, id, options)
    {
        options = options ? options : {};
        var content          = options.content || '';
        var postData         = options.postData || '';
        var onResizeEnd      = options.onResizeEnd || function(){};
        var onResize         = options.onResize || function(){};
        var onComplete       = options.onComplete || function(){};
        var onMaximize       = options.onMaximize || function(){};
        var onUnMaximize     = options.onUnMaximize || function(){};
        var onClose          = options.onClose || function(){};
        var onCreated        = options.onCreated || function(){};
        var resizable        = options.resizable ? true : false;
        var removeConfirm    = options.removeConfirm ? true : false;
        var url              = options.url || false;
        var closeChild       = $defined(options.closeChild) ? options.closeChild : false;
        var btnReduce        = $defined(options.btnReduce) ? options.btnReduce : true;
        var btnMaximize      = $defined(options.btnMaximize) ? options.btnMaximize : true;
        var btnClose         = $defined(options.btnClose) ? options.btnClose : true;
        var modal            = $defined(options.modal) ? options.modal : false;
        var statusbar        = options.statusbar || '';
        var className        = options.className ? ' '+options.className : '';
        var childOf          = $(options.childOf) ? options.childOf : 'desktop';
        var posx             = options.posx || 0;
        var posy             = options.posy || 0;
        var minx             = options.minx || 200;
        var miny             = options.miny || 100;
        var maxx             = options.maxx || function(){
            var index = OS.windows.xref.indexOf(id);
            var posx = OS.windows.info[index][2];
            return OS.dw - posx;
        };
        var maxy             = options.maxy || function(){
            var index = OS.windows.xref.indexOf(id);
            var posy = OS.windows.info[index][3];
            return OS.dh - posy;
        };
        
        if($(id))
        {
            this.restoreWindow(id);
            this.systemInfo('<b>'+id.substr(0, 16)+'</b> exists !');
            return true;
        }
        
        // Window structure
        var item = new Element('div', {
            'id': id,
            'class': 'window'+className,
            'styles': {
                'z-index': 1,
                'position': 'absolute',
                'visibility': 'hidden'
            }
        });
        var left_bar = new Element('div', {
            'id': id+'_left_bar',
            'class': 'window_left_bar window_bar'
        });
        var top_bar = new Element('div', {
            'id': id+'_top_bar',
            'class': 'window_top_bar window_bar'
        });
        var right_bar = new Element('div', {
            'id': id+'_right_bar',
            'class': 'window_right_bar window_bar'
        });
        var bottom_bar = new Element('div', {
            'id': id+'_bottom_bar',
            'class': 'window_bottom_bar window_bar'
        });
        
        // Content
        var win_content = new Element('div', {
            'id': id+'_content',
            'class': 'window_content',
            'events': {
                'click': function(){
                    OS.updateZindex(id);
                },
                'contextmenu': function(e){
                    e.stopPropagation();
                }
            }
        });
        if($type(content)=='element') {
            content.inject(win_content);
        }
        else {
            win_content.set('html', content);
        }
        
        // Title bar
        var title = new Element('div', {
            'id': id+'_title',
            'class': 'window_title',
            'events': {
                'dblclick': function(e){
                    e.preventDefault();
                    OS.maximizeWindow(id, onMaximize, onUnMaximize);
                },
                'contextmenu': function(e){
                    e.stopPropagation();
                }
            }
        });
        var title_txt = new Element('div', {
            'id': id+'_title_txt',
            'html': label,
            'class': 'window_title_txt'
        });
        var title_buttons = new Element('div', {
            'class': 'window_buttons'
        });
        
        // Buttons        
        if(btnClose)
        {
            var btn_close = new Element('div', {
                'class': 'window_btn_close',
                'title': 'Close',
                'html': '',
                'events': {
                    'click': function(e){
                        e.preventDefault();
                        OS.removeWindow(id, {
                            'onClose': onClose,
                            'removeConfirm': removeConfirm,
                            'closeChild': closeChild,
                            'modal': modal
                        });
                    }
                }
            });
            btn_close.inject(title_buttons);
        }
        
        if(btnMaximize)
        {
            var btn_max = new Element('div', {
                'id': id+'_max',
                'class': 'window_btn_max',
                'title': 'Maximize/Unmaximize',
                'html': '',
                'events': {
                    'click': function(e){
                        e.preventDefault();
                        OS.maximizeWindow(id, onMaximize, onUnMaximize);
                    }
                }
            });
            btn_max.inject(title_buttons);
        }
        
        if(btnReduce)
        {
            var btn_reduce = new Element('div', {
                'class': 'window_btn_reduce',
                'title': 'Reduce',
                'html': '',
                'events': {
                    'click': function(e){
                        e.preventDefault();
                        OS.reduceWindow(id);
                    }
                }
            });
            btn_reduce.inject(title_buttons);
        }
        
        // Modal Window
        if(modal)
        {
            modal = $(modal) ? modal : 'desktop';
            this.taskbar.hide(modal);
            var modal_elem = $(modal);
            var modal_hide = modal_elem.getLast('.window_modal_hide');
            if(!modal_hide)
            {
                modal_hide = new Element('div',{
                    'class': 'window_modal_hide',
                    'events': {'click': function()
                    {
                        OS.updateZindex('', modal_elem);
                        OS.updateZindex(id, item);
                    }}
                });
                modal_hide.inject(modal_elem);
            }
            item.store('modal', modal);
            this.updateZindex('', modal_hide);
            this.updateZindex(id, item);
        }
        
        // Inject window elements in the item
        title_txt.inject(title);
        title_buttons.inject(title);
        title.inject(item);
        win_content.inject(item);
        item.inject(this.desktop)
        this.taskbar.add(id, label);
       
        // Bars 
        if(statusbar)
        {
            var status_bar = new Element('div', {
                'id': id+'_status_bar',
                'class': 'window_status_bar',
                'html': statusbar
            });
            status_bar.inject(item);
            win_content.style.bottom = status_bar.getStyle('height');
        }
        left_bar.inject(item);
        top_bar.inject(item);
        right_bar.inject(item);
        bottom_bar.inject(item);
        
        // Crossed references
        var index = this.windows.xref.push(id) - 1;
        this.windows.unmax[index] = [];
        this.windows.parents[index] = childOf;
        
        // Dynamic content
        if(url)
        {
            var req = new Request.HTML(
            {
                'url': this.path+url+'?session_id='+session_id,
                'method': 'post',
                'update': id+'_content',
                'evalScripts': true,
                'onComplete': function(){
                    onComplete();
                }
            }).send('id='+id+postData);
        }
        item = $(id);
        this.windows.info[index] = this.windowInformation(id, item);
        
        //Window size
        var width = options.width ? options.width : options.minx ? options.minx : $(id+'_content').scrollWidth ?
            $(id+'_content').scrollWidth +
            $(id+'_left_bar').getStyle('width').toInt()*2 : this.desktop.scrollWidth / 2;
        
        var height = options.height ? options.height : options.miny ? options.miny : $(id+'_content').scrollHeight ?
            $(id+'_content').scrollHeight +
            $(id+'_title').getSize().y +
            $(id+'_bottom_bar').getStyle('height').toInt()*2 : this.desktop.scrollHeight / 2;
        
        this.windows.info[index][0] = width;
        this.windows.info[index][1] = height;
        item.style.width = width+'px';
        item.style.height = height+'px';
        
        // Position
        var parentindex = this.windows.xref.indexOf(childOf);
        var tbH = title.getSize().y;
        // Automatic position: one half of width and one fifth of height of the parent
        if(!posx && !posy)
		{
            //this.setWinPosQuad(item, id, parentindex);
            var x = (this.windows.info[parentindex][0] / 2) - (width / 2) + this.windows.info[parentindex][2];
            var y = (this.windows.info[parentindex][1] - height) / 5 + this.windows.info[parentindex][3];
            x = x.round();
            y = y.round();
            this.setWinPos(item, id, index, x, y, this.windows.info[parentindex], tbH, false);
        }
        // Manual position
        else
		{
            OS.windows.info[index][2] = posx;
            OS.windows.info[index][3] = posy;
        }
        
        OS.fx.move_in_nw(item, index);
        
        // Move window
        item.drag = new Drag.Move(item, {
            'handle': id+'_title',
            'onBeforeStart': function(item) {
                OS.updateZindex(id, item);
            },
            'onComplete': function() {
                OS.windows.info[index] = OS.windowInformation(id, item);
                OS.setWinPos(item, id, index, OS.windows.info[index][2], OS.windows.info[index][3], [OS.dw, OS.dh, 0, 0], tbH, false);
                OS.icons.parents.each(function(v,k) {
                    if(v===id) {
                        var icon_id = OS.icons.xref[k];
                        OS.icons.info[k] = OS.iconInformation(icon_id);
                    }
                });
            }
        });
        
        //Resizable window if it's an application (OS.modules.run) or if it's been requested explicitly
        if((OS.modules.run[item.id]!==undefined && OS.modules.run[item.id].resizable) || resizable)
        {
            var btn_resize = new Element('div', {
                'id': id+'_resize',
                'class': 'button_resize'
            });
            btn_resize.inject(item);
            var resize_complete = function(item)
            {
                OS.windows.info[index] = OS.windowInformation(id, item);
                OS.icons.parents.each(function(v,k){
                    if(v===id)
                    {
                        var icon_id = OS.icons.xref[k];
                        OS.icons.info[k] = OS.iconInformation(icon_id);
                    }
                });
                onResizeEnd();
            }
            // Resizing
            // Bottom right corner
            item.makeResizable({
                'handle': id+'_resize',
                'modifiers': {'x': 'width', 'y': 'height'},
                'limit': {'x': [minx, maxx], 'y': [miny, maxy]},
                'onComplete': resize_complete,
                'onDrag': onResize
            });
            // Right bar
            $(id+'_right_bar').setStyle('cursor', 'e-resize');
            new Drag(item,
            {
                'handle': id+'_right_bar',
                'modifiers': {'x': 'width', 'y': false},
                'limit': {'x': [minx, maxx]},
                'onComplete': resize_complete,
                'onDrag': onResize
            });
            // Bottom bar
            $(id+'_bottom_bar').setStyle('cursor', 's-resize');
            new Drag(item,
            {
                'handle': id+'_bottom_bar',
                'modifiers': {'x': false, 'y': 'height'},
                'limit': {'y': [miny, maxy]},
                'onComplete': resize_complete,
                'onDrag': onResize
            });
        }
        
		onCreated();
        
        // Put window on top of desktop
        setTimeout(function(){
            OS.updateZindex(id, item);
        }, 10);
        
        return id;
    },

/**********************************************
 * Method: removeWindow
 * Remove a window and kill the process if the application has a path
 * 
 * Parameters:
 *     id
 *     options - onClose, removeConfirm, closeChild
 **********************************************/
    'removeWindow': function(id, options)
    {
        if($(id))
        {
            var item = $(id);
            var index = this.windows.xref.indexOf(id);
            var parentid = this.windows.parents[index];
            if(parentid==='desktop')
            {
                this.new_window_pos -=1;
            }
            
            if(options)
            {
                // Confirmation on window close
                if(options.removeConfirm)
                {
                    this.infoWindow('Alert !', id+'_removeConfirm', {
                        'content': 'Do you want to <b>Quit"'+$(id+'_title_txt').innerHTML+'" ?</b>',
                        'childOf': id,
                        'actions': {'yes': function(){ OS.removeWindow(id); }, 'no': function(){}}
                    });
                    return true;
                }
                // Check if there are children
                if(options.closeChild)
                {
                    var children = this.windowChild(id);
                    if(children)
                    {
                        this.infoWindow('Alert !', id+'_removeConfirm', {
                            'content': 'Do you want to <b>Quit All Windows Attached To "'+$(id+'_title_txt').innerHTML+'" ?</b>',
                            'childOf': id,
                            'actions': {
                                'yes': function(){
                                    children.each(function(i){
                                        OS.removeWindow(i);
                                    });
                                    OS.removeWindow(id);
                                },
                                'no': function(){
                                    children.each(function(i){
                                        OS.updateZindex(i);
                                    });
                                }}
                        });
                        return true;
                    }
                }
                // Function to execute on window close
                if($type(options.onClose)==='function')
                {
                    options.onClose();
                }
            }
            
            // Modal Window
            var modal = item.retrieve('modal');
            if($(modal))
            {
                var modal_hide = $(modal).getLast('.window_modal_hide');
                if(modal_hide)
                {
                    modal_hide.dispose();
                }
                this.updateZindex(id, item);
                this.taskbar.hide(modal);
            }
            
            // Remove window
            var content = $(id+'_content');
            content.empty();
            
            OS.fx.move_out_nw(item, index);
            
            this.windows.info = this.unset(this.windows.info, index);
            this.windows.xref = this.unset(this.windows.xref, index);
            this.windows.unmax = this.unset(this.windows.unmax, index);
            this.windows.parents = this.unset(this.windows.parents, index);
            this.taskbar.rm(id);
            
            //var req = new Request(
            //{
            //    'url': this.path+'Kernel/Server/syscall.php?session_id='+session_id,
            //    'method': 'post'
            //}).send('req=killProcess&id='+id);
            
            if(this.modules.run[id])
            {
                delete this.modules.run[id];
            }
        }
        return true;
    },

/**********************************************
 * Method: windowChild
 * Check if the window has child(ren) and return
 * it/them in an array.
 * 
 * Parameters:
 *     id - id of the window
 *
 * Returns:
 *   array - array of children id's if any, or false
 **********************************************/
    'windowChild': function(id, children)
    {
        children = children ? children : [];
        var child, index;
        this.windows.parents.each(function(i, index)
        {
            if(i===id) {
                child = OS.windows.xref[index];
                children.push(child);
                children = OS.windowChild(child, children);
            }
        });
        
        if(children.length>0) {
            return children;
        }
        else {
            return false;
        }
    },
    
/**********************************************
 * Function setWinPosQuad
 * Set Window Position as N/S/E/W.
 * Not used anymore atm.
 * 
 * Parameters:
 *     item - Element to position
 *     id - id of the Element
 *     parentindex - index of the parent, could be guessed too
 **********************************************/
    'setWinPosQuad': function(item, id, parentindex)
    {
        var index = this.windows.xref.indexOf(id);
        var dim_parent = this.windows.info[parentindex];
        var tbH = $(id+'_title').getSize().y/2;
        var x,y;
        if(parentindex===0)
        {
            switch(this.new_window_pos)
            {
                case 0:
                    //bottom left
                    x = dim_parent[2] + 5;
                    y = dim_parent[1] - this.windows.info[index][1] - tbH;
                    this.new_window_pos +=1;
                    break;
                case 1:
                    //top left
                    x = dim_parent[2] + 5;
                    y = dim_parent[3] + tbH;
                    this.new_window_pos +=1;
                    break;
                case 2:
                    //top right
                    x = dim_parent[0] - this.windows.info[index][0] - tbH;
                    y = dim_parent[3] + tbH;
                    this.new_window_pos +=1;
                    break;
                case 3:
                    //bottom right
                    x = dim_parent[0] - this.windows.info[index][0] - tbH;
                    y = dim_parent[1] - this.windows.info[index][1] - tbH;
                    this.new_window_pos = 0;
                    break;
                default:
                    x = dim_parent[2] + tbH;
                    y = dim_parent[3] + tbH;
                    break;
            }
        }
        else
        {
            x = dim_parent[2];
            y = dim_parent[3];
        }
        this.setWinPos(item, id, index, x, y, dim_parent, tbH, false);
    },

/**********************************************
 * Method: setWinPos
 * Set Window Position Recursively
 *
 * Parameters:
 *     item - Element to position
 *     id - id of the Element
 *     index - index of the element in OS.windows.info array
 *     x - horizontal base position
 *     y - vertical base position
 *     dim_parent - parent information in the same format than OS.windows.info
 *     tbH - height of the window title
 *     neg - direction of the positionning, if set to true, the windows will go back on x and y
 **********************************************/
    'setWinPos': function(item, id, index, x, y, dim_parent, tbH, neg)
    {
        var check = 0;
        var info_all = this.windows.info;
        var w = this.windows.info[index][0];
        var h = this.windows.info[index][1];
        var dw = this.dw;
        var dh = this.dh;
        // Square (the window title) to check intersection
        var c1 = [x, y, (x+w), (y+tbH-2)];
        
        // Check if the window is being position at a place it connot be repositionned (outside desktop)
        if(y<0){y=0;}
        else if(y>dh-tbH) {y=dh-h-tbH}
        if(x<0-w+tbH){x=0;}
        else if(x>dw-tbH) {x=dw-w-tbH}
        
        // Check if any window is under current window
        this.windows.info.each(function(v,i){
            var wi = info_all[i][0];
            var xi = info_all[i][2];
            var yi = info_all[i][3];
            var window_id = OS.windows.xref[i];
            var c2 = [xi, yi, (xi+wi), (yi+tbH-2)];
            // If the window overlap any other window title
            if(
               window_id!==id &&
               window_id!=='desktop' &&
               yi!==undefined &&
               OS.squareIntersect(c1, c2)
            ){
                // Decalage
                if((x+w+tbH)>dw && (y+h+tbH)>dh && !neg) {neg=true;}
                if(!neg){
                    x = xi+tbH;
                    y = yi+tbH;
                }
                else{
                    x = xi-tbH;
                    y = yi-tbH;
                }
                check +=1;
            }
        });
        // If check is finished, then place the window a the free position found
        if(check===0){
            this.windows.info[index][2] = x;
            this.windows.info[index][3] = y;
            item.style.left = x+'px';
            item.style.top = y+'px';
            return true;
        }
        // Search for free position (Recursion)
        else{
            this.setWinPos(item, id, index, x, y, dim_parent, tbH, neg);
        }
        return true;
    },

/**********************************************
 * Method: toggleWindow
 * Toggle from reduced and restored mode for windows
 * 
 * Parametres:
 *     id - id of the window
 **********************************************/
    'toggleWindow': function(id)
    {
        var link = $(id+'_reduced');
        if(link.hasClass('reduced'))
        {
          OS.restoreWindow(id);  
        }
        else
        {
           OS.reduceWindow(id); 
        }
    },

/**********************************************
 * Method: reduceWindow
 * 
 * Parametres:
 *     id - id of the window
 **********************************************/
    'reduceWindow': function(id)
    {
        var item = $(id);
        var link = $(id+'_reduced');
        if(item.style.zIndex.toInt()===(OS.zindex_id-1))
        {
            link.addClass('reduced');
            this.fx.move_in_elem(item, link);
        }
        this.updateZindex(id, item);
    },

/**********************************************
 * Method: restoreWindow
 * Retore reduced window and update taskbar
 * 
 * Parametres:
 *     id - id of the window
 **********************************************/
    'restoreWindow': function(id)
    {
        var item = $(id);
        var link = $(id+'_reduced');
        var index = this.windows.xref.indexOf(id);
        link.removeClass('reduced');
        this.fx.move_out_elem(item, index, link);
        setTimeout(function(){
            OS.updateZindex(id, item);
        }, 10);
    },

/**********************************************
 * Method: maximizeWindow
 * Maximize Window
 * 
 * Parametres:
 *     id - id of the window
 *     onMaximize - function to execute when the window is maximezed
 *     onUnMaximize - function to execute when the window is unmaximized
 **********************************************/
    'maximizeWindow': function(id, onMaximize, onUnMaximize)
    {
        var item = $(id);
        var btn = $(id+'_max');
        var title = $(id+'_title');
        var d_size = {};
        var index = this.windows.xref.indexOf(id);
        
        this.windows.unmax[index] = this.windowInformation(id, item);
        this.windows.info[index] = [this.dw, this.dh, 0, 0];
        OS.fx.maximize_window(item, onMaximize);
        item.drag.detach();
        
        if(btn) {
            btn.style.backgroundPosition = '0 -45px';
            btn.removeEvents('click');
            btn.set('events', {
                'click': function(e){
                    e.preventDefault();
                    OS.unmaximizeWindow(id, onMaximize, onUnMaximize);
                }
            });
        }
        title.removeEvents('dblclick');
        title.set('events', {
            'dblclick': function(e){
                e.preventDefault();
                OS.unmaximizeWindow(id, onMaximize, onUnMaximize);
            }
        });
    },

/**********************************************
 * Method: unmaximizeWindow
 * Unmaximize Window
 * 
 * Parametres:
 *     id - id of the window
 *     onMaximize - function to execute when the window is maximezed
 *     onUnMaximize - function to execute when the window is unmaximized
 **********************************************/
    'unmaximizeWindow': function(id, onMaximize, onUnMaximize)
    {
        var item = $(id);
        var btn = $(id+'_max');
        var title = $(id+'_title');
        var index = this.windows.xref.indexOf(id);
        var info = this.windows.unmax[index];
        
        this.windows.info[index] = info;
        item.drag.attach();
        OS.fx.unmaximize_window(item, info, onUnMaximize);
        
        title.removeEvents('dblclick');
        title.set('events', {
            'dblclick': function(e){
                e.preventDefault();
                OS.maximizeWindow(id, onMaximize, onUnMaximize);
            }
        });
        if(btn) {
            btn.style.backgroundPosition = '0 -30px';
            btn.removeEvents('click');
            btn.set('events', {
                'click': function(e){
                    e.preventDefault();
                    OS.maximizeWindow(id, onMaximize, onUnMaximize);
                }
            });
        }
    },

/**********************************************
 * Method: removeAllWindows
 * Remove All Windows
 *
 * Parametres:
 *     except - Array of window ids to ignore
 **********************************************/
    'removeAllWindows': function(except)
    {
        except = (except) ? except : [];
        this.windows.xref.each(function(id)
        {
            if(id!=='desktop' && OS.isNotPresent(except, id))
            {
                OS.removeWindow(id);
            }
        });
    },

/**********************************************
 * Method: windowInformation
 * Windows Information (positions and dimensions)
 * Detecter les dimensions et positions des fenetres
 **********************************************/
    'windowInformation': function(id, item)
    {
        if(id)
        {
            if(!item)
            {
                item = $(id);
            }
            var index = this.windows.xref.indexOf(id);
            var par = this.windows.parents[index];
            var coord = item.getCoordinates(this.desktop);
            return [coord.width, coord.height, coord.left, coord.top];
        }
        return true;
    },

/**********************************************
 * Method: updateZindex
 * Increase zIndex of a window/element.
 * If it's a window, flags it 'selected' in the taskbar.
 * If the element is reduced, it's restored.
 * Return false if the object does not exist
 *
 * Parametres:
 *     id - id
 *     item - item
 **********************************************/
    'updateZindex': function(id, item)
    {
        if(!item)
        {
            item = $(id);
        }
        if(item)
        {
            if(item.style.zIndex != (this.zindex_id - 1) || item.style.zIndex.toInt()===1)
            {
                item.style.zIndex = this.zindex_id;
                this.zindex_id +=1;
                if(item.hasClass('window'))
                {
                    var reduced = $(id+'_reduced');
                    var previous_window = $(this.selected_window+'_reduced');
                    if(previous_window) {previous_window.removeClass('selected');}
                    if(reduced) {reduced.addClass('selected');}
                    this.selected_window = id;
                }
            }
            return true;
        }
        return false;
    },

/**********************************************
 * Group: Dialog windows
 **********************************************/
/**********************************************
 * Method: infoWindow
 * 
 * Parametres:
 *   title - Title
 *   id - Id
 *   options - see Options
 *
 * Options:
 *   content - String that goes in the window
 *   posx - Horizontal position
 *   posy - Vertical position
 *   width - Width of the window, Default: 330px
 *   height - Height of the window
 *   childOf - Parent of the window, Default: desktop
 *   actions - Dictionnary of button's label and action. {'label of button': function(){//action goes here}}
 *   btn_default - Default button selected. Value: 'label of button', Default: first button
 *   onClose - Function to fire when closed
 *   modal - id of the attached window/element, if set to false, window is not modal. Default: childOf
 **********************************************/
    'infoWindow': function(title, id, options)
    {
        options = options || {};
        var width = options.width || 330;
        var height = options.height || false;
        var content = options.content || '';
        var childOf = options.childOf || 'desktop';
        var onClose = options.onClose || function(){};
        var actions = options.actions ? new Hash(options.actions) : new Hash({'ok': function(){}});
        var btn_default = options.btn_default || false;
        var modal = $defined(options.modal) ? options.modal : $(childOf) ? childOf : 'desktop';
        var btn=[], i=0, posx, posy, btn_elem, selected_button;

        // Set window positions
        if(!options.posx && !options.posy && $(childOf))
        {
            var index = OS.windows.xref.indexOf(childOf);
            if(index!=-1)
            {
                posx = (OS.windows.info[index][0] / 2) - (width / 2) + OS.windows.info[index][2];
                posy = OS.windows.info[index][1] / 5 + OS.windows.info[index][3];
            }
        }
        else
        {
           posx = options.posx;
           posy = options.posy;
        }
        
        // Base element
        var html = new Element('div', {
            'html': content+'<br/>'
        });

        // Creation of the buttons
        actions.each(function(action, label){
            // Button Creation
            btn_elem = new Element('button', {
                'id':  id+'_button_info_'+i,
                'class': 'button_info',
                'html': label,
                'events': {
                    'click': function() {
                        if($type(action)==='function') {
                            action();
                        }
                        else {
                            eval(action);
                        }
                        OS.removeWindow(id+'_info');
                    }
                }
            });
            btn_elem.inject(html);
            // Set default button if defined
            if(label==btn_default || i==0) {
                selected_button = btn_elem;
            }
            // Counter
            i+=1;
        });
        
        // Creation of the window
        OS.createWindow(title, id+'_info',
        {
            'content': html,
            'posx': posx,
            'posy': posy,
            'width': width,
            'height': height,
            'childOf': childOf,
            'onClose': onClose,
            'btnMaximize': false,
            'btnReduce': false,
            'modal': modal,
            'className': 'window_infoWindow'
        });
 
        // Select the default selected button
        if(selected_button) {
            selected_button.addClass('button_default');
            selected_button.focus();
        }
    },

/**********************************************
 * Method: promptWindow
 * 
 * Parametres:
 *   title - Title
 *   id - Id
 *   options - see Options
 *
 * Options:
 *   content - String that goes in the window
 *   posx - Horizontal position
 *   posy - Vertical position
 *   width - Width of the window, Default: 330px
 *   height - Height of the window
 *   childOf - Parent of the window, Default: desktop
 *   action - Action to fire if ok is pressed, the value of the input is passed as argument.
 *   onClose - Function to fire when closed
 *   modal - id of the attached window/element, if set to false, window is not modal. Default: childOf
 **********************************************/
    'promptWindow': function(title, id, options)
    {
        //FIXME: USE GUI ELEMENTS PLEASE (INPUT FOR INSTANCE)
        options = options ? options : {};
        var width = options.width ? options.width : 330;
        var height = options.height ? options.height : false;
        var onClose = options.onClose ? options.onClose : function(){};
        var content = options.content? options.content : '';
        var childOf = options.childOf ? options.childOf : 'desktop';
        var action = options.action ? options.action : function(){};
        var modal = $defined(options.modal) ? options.modal : $(childOf) ? childOf : 'desktop';
        var btn = [];
        var i=0;
        var posx, posy, btn_ok, btn_cancel;

        // Set window positions
        if(!options.posx && !options.posy && $(childOf))
        {
            var index = OS.windows.xref.indexOf(childOf);
            if(index!=-1)
            {
                posx = (OS.windows.info[index][0] / 2) - (width / 2) + OS.windows.info[index][2];
                posy = OS.windows.info[index][1] / 5 + OS.windows.info[index][3];
            }
        }
        else
        {
           posx = options.posx;
           posy = options.posy;
        }
        
        // Base element
        var html = new Element('div', {
            'html': content+'<br/>'
        });

        // Input element
        var input = new Element('input', {
            'id': id+'_prompt_input',
            'type': 'text'
        });
        input.inject(html);
        html.set('html', html.get('html')+'<br/>');

        // Creation of the buttons
        btn_ok = new Element('button', {
            'id':  id+'_button_prompt_ok',
            'class': 'button_prompt',
            'html': 'ok',
            'events': {
                'click': function() {
                    if($type(action)==='function') {
                        var input_value = $(id+'_prompt_input').get('value');
                        action.run(input_value);
                    }
                    OS.removeWindow(id+'_prompt');
                }
            }
        });
        btn_cancel = new Element('button', {
            'id':  id+'_button_prompt_cancel',
            'class': 'button_prompt',
            'html': 'cancel',
            'events': {
                'click': function() {
                    OS.removeWindow(id+'_prompt');
                }
            }
        });

        btn_ok.inject(html);
        btn_cancel.inject(html);
        
        // Creation of the window
        OS.createWindow(title, id+'_prompt',
        {
            'content': html,
            'posx': posx,
            'posy': posy,
            'width': width,
            'height': height,
            'childOf': childOf,
            'onClose': onClose,
            'btnMaximize': false,
            'btnReduce': false,
            'modal': modal,
            'className': 'window_promptWindow'
        });
 
        // Select the input by default
        var first = $(id+'_prompt_input');
        if(first) {
            first.focus();
        }
    }
});

