<?php
/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.4
        Web Desktop System
*/
/**********************************************
 * File: Programs.class.php
 * Class: Programs
 * Extends ProtocolBuffers
 **********************************************/
class Programs extends ProtocolBuffers
{
/**********************************************
 * Method: arrayToJson
 * Transform an object into formated JSON
 *
 * Parameters:
 *   cols - formated Cols, must be object's methods
 *   obj - Object
 *
 * Returns:
 *  Array in JSON
 **********************************************/
	public function arrayToJson($cols, $obj)
	{
        return json_encode(array(
            'cols'=>$cols,
            'rows'=>$obj
        ));
	}
    
    public function jsonToArray($string)
    {
        $out = array();
        $obj = json_decode(stripslashes($string));
        if($obj)
        {
            foreach($obj->rows as $i => $row)
            {
                $raw_out = array();
                foreach($row as $j => $col)
                {
                    $col_title = $obj->cols[$j];
                    $raw_out[$col_title] = $col;
                }
                $out[] = $raw_out;
            }
        }
        return $out;
    }

/**********************************************
 * Method: pb_apps_get
 * Get applications object from PB data and return
 * an associative array with values
 *
 * Returns:
 *  Apps associative array
 **********************************************/
    public function pb_apps_get()
    {
        $out = array(
            'cols' => array('id', 'label', 'path', 'state', 'depends'),
            'rows' => array()
        );
        $string = $this->pb_get('apps.pb');
        $this->pb_require('apps');
        $apps = new Apps();
        $apps->parseFromString($string);
        
        for($i=0; $i<$apps->apps_size(); $i++)
        {
            $app = $apps->apps($i);
            // Dependances
            $depends = array();
            for($j=0; $j<$app->depends_size(); $j++)
            {
                array_push($depends, $app->depends($j)->id());
            }
            // State
            switch($app->state())
            {
                case Application_Status::RC:
                    $state = 'rc';
                    break;
                case Application_Status::II:
                    $state = 'ii';
                    break;
                default:
                    $state = 'rc';
            }
            $out['rows'][] = array(
                $app->id(),
                $app->label(),
                $app->path(),
                $state,
                $depends
            );
        }
        return $out;
    }

/**********************************************
 * Method: pb_apps_set
 * Write a programs array to PB
 *
 * Parameters:
 *   json - string that contains apps array
 *
 * Returns:
 *  true on succes, false on error
 **********************************************/
    public function pb_apps_set($json)
    {
        $this->pb_require('apps');
        $apps = new Apps();
        $array = $this->jsonToArray($json);
        $length = count($array);
        for($i=0; $i<$length; $i++)
        {
            // State
            switch($array[$i]['state'])
            {
                case 'rc':
                    $state = Application_Status::RC;
                    break;
                case 'ii':
                    $state = Application_Status::II;
                    break;
                default:
                    $state = Application_Status::RC;
            }

            // Force installation of Apps management !
            if($array[$i]['id']==='programs') {
                $state = Application_Status::II;
            }
            
            // Set App infos
            $app = $apps->add_apps();
            $app->set_id($array[$i]['id']);
            $app->set_label($array[$i]['label']);
            $app->set_path($array[$i]['path']);
            $app->set_state($state);
            $depend = $app->add_depends();
            $depend->set_id($array[$i]['depends']);
        }
        return $this->pb_set($apps, 'apps.pb');
    }

/**********************************************
 * Method: buildModulesList
 * Build a modules list from JS files
 *
 * Returns:
 *  Request formated message with buildRequest
 **********************************************/
    public function buildModulesList()
    {
        // Output array
        $out= array(
            'cols' => array('id', 'label', 'path', 'state', 'depends'),
            'rows' => array()
        );
        // Get Files list
        $files = glob($this->path.'/programs/*/*.js', GLOB_ERR);
        
        if($files){
            foreach($files as $file)
            {
                // Get module's file content
                $content = file_get_contents($file);
                
                //id
                $pattern = "/'id'\:[ |]'(.*)'/";
                preg_match($pattern, $content, $m);
                $id = $m[1] ? $m[1] : basename($file);
                
                //label
                $pattern = "/'label'\:[ |]'(.*)'/";
                preg_match($pattern, $content, $m);
                $label = $m[1] ? $m[1] : basename($file);
                
                //path
                $path = str_replace($this->path.'/', '', $file);
                
                //state
                $state = 'ii';
                
                //depends
                $pattern = "/'depends'\: ?\['(.*)'\]/";
                preg_match($pattern, $content, $m);
                $depends = $m[1] ? $m[1] : '';
                
                // Add to output
                $out['rows'][] = array(
                    $id, $label, $path, $state, $depends
                );
            }
            return $this->pb_apps_set(json_encode($out));
        }
        else{
            return __FUNCTION__.': Permissions denied in '.$this->path.'/programs/';
        }
    }
}
?>