<?php
/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.4
        Web Desktop System
*/
include_once('Kernel/config.php');
/**********************************************
 * Class: server
 * The "server" part of TakOS, handle session,
 * file management, process.
 * Must stay ligth enough, please delegate in modules.
 * This is a singleton class.
 **********************************************/
class os
{
/**********************************************
 * Properties: Properties
 * $os_session - object session name
 * prox - array of process
 * next_pid - iterated value to set process ID's
 * user - username
 **********************************************/
    static public $path;
    static public $servers_file='/Data/servers.list';
    public $session_id='';
	public $user;
    public $version='1.4';
	private $proc;
	private $next_pid;
	private static $os_session='os'; // FIXME: Find a way to match $os_session from config.php

/**********************************************
 * Constructor: __construct
 * Start the session and make sure proc exists.
 * Private because this is a singleton class.
 **********************************************/
	private function __construct($session_id)
	{
        if(!empty($session_id) && preg_match('/^[0-9a-fA-F]*$/', $session_id)){
            session_id($session_id);
            session_start();
            $this->session_id=$session_id;
        }
        else{
            session_start();
            $this->session_id = session_id();
            $this->next_pid = 1;
        }
        if(empty($this->proc)){
            $this->proc = array();
        }
	}

/**********************************************
 * Destructor: __destruct
 * If object instance is valid, save it
 **********************************************/
	function __destruct()
	{
        if(!empty($this->user)){
           $this->saveInstance();
        }
        ob_flush();
	}

/**********************************************
 * Method: getKernel
 * Get or create an instance of the singleton class
 **********************************************/
	public static function getInstance($session_id='')
	{
        $c = __CLASS__;
        $obj_new = new $c($session_id);
        $obj_session = unserialize($_SESSION[self::$os_session]);
        
        // Get object from session
        if(is_object($obj_session)){
    		return $obj_session;
        }
        // Use a new object
        elseif(is_object($obj_new)){
            return $obj_new;
        }
	}

/**********************************************
 * Group: Private
 **********************************************/
/**********************************************
 * Method: saveInstance
 **********************************************/
	private function saveInstance()
	{
		$_SESSION[self::$os_session] = serialize($this);
	}

/**********************************************
 * Method: getPid
 * Get a PID for a new process
 **********************************************/
	private function getPid()
	{
		$pid = str_pad($this->next_pid, 3, '0', STR_PAD_LEFT);
		$this->next_pid++;
		return $pid;
	}

/**********************************************
 * Method: getModulePos
 * Get the module position in the proc array
 *
 * Parameters:
 *   id - id of the proc to search
 **********************************************/
	private function getModulePos($id)
	{
		foreach($this->proc as $pos => $process)
		{
			if($process['id']==$id)
			{
				return $pos;
			}
		}
	}
	
/**********************************************
 * Group: Public
 **********************************************/
/**********************************************
 * Method: logout
 **********************************************/
	public function logout()
	{
        $_SESSION=array();
        session_destroy();
        $this->logged=false;
    }

/**********************************************
 * Group: Grid/Array
 *
 * Example:
 *   array1 = array(
 *       array(1,2,3), // row 0
 *       array(4,5,6), // row 1
 *       array(7,8,9) // row 2
 *   );
 *
 *   // 1,4,7 is col 0
 *   // 2,5,8 is col 1
 *   // 3,6,9 is col 2
 **********************************************/
/**********************************************
 * Method: getRow
 *
 * Properties:
 *   array - Multidimensional array
 *   row - Row number to get, starting from 0
 **********************************************/
	public function getRow($array, $row)
	{
        $len = count($array);
        for($i=0; $i<$len; $i++){
            if($i==$row){
                return $array[$i];
            }
        }
        return array();
    }

/**********************************************
 * Method: getColumn
 *
 * Properties:
 *   array - Multidimensional array
 *   col - Column number to get, starting from 0
 **********************************************/
	public function getColumn($array, $col)
	{
        $out = array();
        $len = count($array);
        for($i=0; $i<$len; $i++){
            if(is_array($array[$i]) && count($array[$i])>$col) {
                array_push($out, $array[$i][$col]);
            }
        }
        return $out;
    }

/**********************************************
 * Method: getDiagonal
 *
 * Properties:
 *   array - Multidimensional array
 *   isPositive - By default, it takes the positive diagonal.
 **********************************************/
	public function getDiagonal($array, $isPositive=true)
	{
        $x=0;
        $out=array();
        if($isPositive){
            $limit=-1;
            $y=count($array)-1;
            $i=-1;
        }
        else{
            $limit=count($array);
            $y=0;
            $i=1;
        }
        for($j=0; $y!=$limit; $j++){
            array_push($out, $array[$y][$x]);
            $x+=1;
            $y+=$i;
        }
        return $out;
    }

/**********************************************
 * Method: setRow
 *
 * Properties:
 *   array - Multidimensional array
 *   row_array - the replacement row
 *   row - Row number to be replaced, starting from 0
 **********************************************/
	public function setRow($array, $row_array, $row)
	{
        $len = count($array);
        for($i=0; $i<$len; $i++){
            if($i==$row){
                $array[$i] = $row_array;
                return $array;
            }
        }
        return $array;
    }

/**********************************************
 * Method: setColumn
 *
 * Properties:
 *   array - Multidimensional array
 *   col_array - the replacement col
 *   col - Column number to get starting from 0
 **********************************************/
	public function setColumn($array, $col_array, $col)
	{
        $len = count($array);
        for($i=0; $i<$len; $i++){
            if(is_array($array[$i]) && count($array[$i])>$col) {
                $array[$i][$col] = $col_array[$i];
            }
        }
        return $array;
    }

/**********************************************
 * Group: Utilities
 **********************************************/

/**********************************************
 * Method: chmod
 * Recursive chmod directories and files, change mod
 * on file only if fmod is specified.
 * 
 * Parameters:
 *      parent - directory or file path relative to caller script (i.e.: programs/admin/)
 *      dmod - mod for directories (i.e.: 0777)
 *      fmod - mod for files (i.e.: 0644), Default: false
 **********************************************/
	public function chmod($parent, $dmod, $fmod=false)
	{
        if(is_dir($parent))
        {
            $old = umask(0000);
            if(@chmod($parent, $dmod))
            {
                umask($old);
                if($handle = opendir($parent))
                {
                    while(($file = readdir($handle)) !== false)
                    {
                        if($file==="." OR $file==="..")
                        {
                            continue;
                        }
                        elseif(is_dir($parent . '/' . $file))
                        {
                            $this->chmod($parent . '/' . $file, $dmod, $fmod);
                        }
                        elseif($fmod)
                        {
                            $old = umask(0000);
                            chmod($parent . '/' . $file, $fmod);
                            umask($old);
                        }
                    }
                    closedir($handle);
                }
            }
            else {
                return "server: chmod: '$parent': Permission denied";
            }
        }
        elseif($fmod)
        {
            $old = umask(0000);
            if(@chmod($parent, $fmod)) {
                umask($old);
            }
            else {
                return "server: chmod: '$parent': Permission denied";
            }
        }
    }

/**********************************************
 * Method: sslInUse
 * Check if SSL is in use throught https
 **********************************************/
	public function sslInUse()
	{
        if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']=='on'){
            return true;
        }
        return false;
	}

/**********************************************
 * Method: sslRedirect
 * Redirect to https
 **********************************************/
	public function sslRedirect()
	{
        $htts_url = 'https://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']);
        header('Status: 301 Moved Permanently', false, 301);
        header("Location: $htts_url");
	}

/**********************************************
 * Method: connectionTest
 **********************************************/
	public function connectionTest($size)
	{
        $content = '';
        $size -= 112; //Actual size of the frame when 'empty'
        for($i=0; $i<$size; $i++){
            $content .= 'x';
        }
        header('Content-type: text/html; charset=utf-8');
        header('Content-length: '.$size);
        header('Cache-Control: no-cache');
        header('Pragma: no-cache');
        header('Expires: 0');
        echo $content;
	}
    
/**********************************************
 * Method: minifyFile
 * Strip comments and whitespace using the PHP
 * JSMin adaptation
 *
 * Parameters:
 *   file - relative path to the file to minify
 *
 * Returns:
 *   rawfile minified
 **********************************************/
	public function minifyFile($file)
	{
        require 'Kernel/ext/jsmin.php';
        return JSMin::minify(file_get_contents($file));
	}

/**********************************************
 * Method: compressFile
 * - Output buffering into GZ Compressed format
 * - Force no-cache
 * - Force UTF-8
 * - Force given content-type
 *
 * Parameters:
 *   type - content-type in HTML format (i.e. text/css)
 **********************************************/
	public function compressFile($type)
	{
        ob_start ("ob_gzhandler");
        header("Content-type: $type; charset: UTF-8");
        header("Cache-Control: must-revalidate");
        header("Expires: ".gmdate("D, d M Y H:i:s",time() + (72 * 60 * 60))." GMT");
	}
    
/**********************************************
 * Method: startProcess
 * Insert process information in the proc array
 *
 * Parameters:
 *   id - id of the new proc
 **********************************************/
	protected function startProcess($id)
	{
		$name = $this->getModuleName();
		$path = $this->getModulePath();
		$process = array(
			'id' => $id, 
			'pid' => $this->getPid(),
			'name' => $id,
			'path' => $path.'/'.$name.'.php'
		);
		array_push($this->proc, $process);
	}

/**********************************************
 * Method: killProcess
 * Remove the process from the proc table
 * FIXME: make this method protected and create a
 * terminateApp, or smth like this
 *
 * Parameters:
 *   id - id of the proc to kill
 **********************************************/
	public function killProcess($id)
	{
		$pos = $this->getModulePos($id);
		unset($this->proc[$pos]);
		sort($this->proc);
	}

/**********************************************
 * Method: listProcess
 * 
 * Returns:
 *   process array
 **********************************************/
	public function listProcess()
	{
		return $this->proc;
	}

/**********************************************
 * Method: findExt
 * Return the extention of a file
 * 
 * Parameters:
 *   file - relative path to file
 *
 * Returns:
 *   extention of the given file
 **********************************************/
	public function findExt($file)
	{
		$file = strtolower($file) ;
		$ext = split("[/\\.]", $file) ;
		$n = count($ext)-1;
		$ext = $ext[$n];
		return $ext;
	}
	
/**********************************************
 * Method: lastSlash
 * Remove the last slash in a string
 *
 * Parameters:
 *   path - string ending with a slash
 *
 * Returns:
 *   String without the last slash
 **********************************************/
	public function lastSlash($path)
	{
		$last_char = substr($path, -1);
		if($last_char=='/')
		{
			return substr($path, 0, strlen($path) - 1);
		}
		return $path;
	}
	
/**********************************************
 * Method: firstSlash
 * Remove the first slash in a string, like lastSlash
 *
 * Parameters:
 *   path - string starting with a slash
 *
 * Returns:
 *   String without the first slash
 **********************************************/
	public function firstSlash($path)
	{
		$first_char = substr($path, 0, 1);
		if($first_char=='/')
		{
			return substr($path, 1, strlen($path));
		}
		return $path;
	}

/**********************************************
 * Method: processPath
 * Check if the new path is inside the project
 * directories structure.
 *
 * Parameters:
 *   arg - path to append
 *   dir_c - current path
 *
 * Returns:
 *   new processed path (basically secure)
 **********************************************/
	public function processPath($arg, $dir_c='')
	{
        $arg = $this->lastSlash($this->firstSlash($arg));
        $dir_c = $this->lastSlash($this->firstSlash($dir_c));
        
		if(strstr($arg, '..'))
		{
			$dirs = ($arg) ? explode('/', $arg) : false;
			$dirs_c = explode('/', $dir_c);
            
			if($dirs[0]=='..')
			{
				if(count($dirs_c) > 0)
				{
					array_pop($dirs_c);
				}
				array_shift($dirs);
			}
			elseif($dirs)
			{
				array_push($dirs_c, $dirs[0]);
				array_shift($dirs);
			}
			else
			{
				return implode('/', $dirs_c);
			}
            
			$arg = (!empty($dirs)) ? implode('/', $dirs) : false;
			$dir_c = implode('/', $dirs_c);
			return $this->processPath($arg, $dir_c);
		}
		else
		{
            $arg = $this->firstSlash($arg);
            $dir_c = $this->lastSlash($dir_c);
			return $dir_c.'/'.$arg;
		}
	}

/**********************************************
 * Method: restartKernel
 * Reload applications list and recreate desktop
 **********************************************/
	public function restartKernel()
	{
	}

/**********************************************
 * Method: resetKernel
 * Reset the programs array in session and
 * return a new instance of the php os object
 *
 * Example:
 *   (start code)
 *   $os = new os($session_id);
 *   $os = $os->resetKernel();
 *   (end)
 **********************************************/
	public function resetKernel()
	{
        //session_regenerate_id(true);
        //$id = session_id();
        
        $req[] = array('exec', 'OS.systemInfo("'.$this->path.'");');
        
            //$this->setSession($username, $os_path);
            //$this->logged=true;
        echo $this->buildRequest($req);
        return new os($id);
	}

/**********************************************
 * Method: getFiles
 * Distribued System: From a file list, give a
 * list of url from different servers.
 *
 * Warning: Files are supposed to be downloaded simultanely
 *
 * Example:
 *   (start code)
 *      $files=array('file1.js', 'file2.jpg', 'file3.css');
 *      $files_new = $os->getFiles($files);
 *   (end)
 *
 *   Properties:
 *      files - Array of files to distribute to the system
 *      
 *  Returns:
 *      files - Array of files with new URL
 **********************************************/
	public function getFiles($files)
	{
        //TODO: Compare $_SERVER[SERVER_ADDR''] and $_SERVER['REMOTE_ADDR']
        $servers_list = $this->path.'/fs/user/Data/servers.list'; ///FIXME: hardcoded
        $files_out = array();
        $file_size = 0;
        $file_nb = count($files);
        $k = 0;
        // Create array of servers from file
        if(is_file($servers_list)) {
            $servers=explode("\n", file_get_contents($servers_list));
        }
        else {
            //FIXME: DL the list
            die('no server list');
            $servers=array();
        }
        $server_nb=count($servers);
        
        // Extract file sizes
        for($i=0; $i<$file_nb; $i++) {
            $file_key = $files[$i];
            $file_size = filesize($path.'/'.$files[$i]);
            $files_size[$file_key] = $file_size;
            $files_size_total += $file_size;
        }
        
        if(is_array($files_size)){

            // Greatest size first
            array_multisort($files_size, SORT_DESC);
        
            // Create file links
            foreach($files_size as $file_name => $file_size) {
                $files_out[] = $servers[$k].'/'.$file_name;
                $k+=1;
                ($k==$server_nb-1) ? $k = 0 : 0;
            }
         }
        
        //TODO: File not found, use local file compressed
        //Kernel/Server/syscall.php?session_id='.$session_id.'&amp;req=includeCss&amp;file='.$style.'
        return $files_out;
	}

/**********************************************
 * Method: readDir
 * List files in a directory, can filter by extention
 * or supported filetypes, wich are:
 * - images (jpg, png, gif)
 *
 * Parameters:
 *   path - path to directory to read
 *   ext - Default=false, extention to list
 *   filetype - Default=false, filetype to list
 *
 * Returns:
 *   array of file pathes
 **********************************************/
	public function readDir($path, $ext=false, $filetype=false)
	{
		$dir = @dir($path);
		if($dir)
		{
			while(($file = $dir->read()) !== false)
			{
				if($file=='.' || $file == '..' || $file == '.bzr')
				{
					continue;
				}
				else
				{
					if(is_dir($dir->path.'/'.$file))
					{
						$dirs[] = $file;
					}
					else
					{
						if($ext==false AND $filetype==false)
						{
							$files[] = $file;
						}
						elseif($ext AND !$filetype)
						{
							if(os::findExt($path.$file)==$ext)
						 	{
						 		$files[] = $file;
						 	}
						}
						elseif($filetype=='image')
						{
							$types_ok = array('image/jpeg', 'image/png', 'image/gif');
							if( list($GLOBALS['largeur'], $GLOBALS['hauteur'], $type) = getimagesize($path.$file)) {
								$type = image_type_to_mime_type($type);
								if (in_array($type, $types_ok)) {
									$files[] = $file;
								}
							}
						}
					}
				}
			}
			$dir->close();
			return array($dirs, $files);
		}
		return false;
	}
    
/**********************************************
 * Method: getBuild
 * Get BZR build (*NIX Only )! 
 * Exec must be permited and Bazaar installed.
 **********************************************/
	public function getBuild()
	{
        $build = exec('bzr version-info |grep "revno"');
        return $build ? $build : '[ERROR]';
	}

/**********************************************
 * Method: startApplication
 **********************************************/
	public function startApplication($id)
	{
		$this->startProcess($id);
	}
    
/**********************************************
 * Group: Request
 **********************************************/
/**********************************************
 * Method: buildRequest
 *
 * See <os.sendRequest>
 *
 * Syntax:
 *   array(
 *     array('type', 'data')
 *     [,array('type', 'data')]
 *   )
 *
 * Types:
 *  apps - List of applications builded with pb_apps_get()
 *  icons - List of icons builded with readDir
 *  exec - raw JS to execute
 *  window_update - update DIV with content
 *  window_info - information window
 *  window_create - create a window
 *  window_remove - remove a window
 *
 *  Data:
 *   apps - json as string
 *   icons - json as string
 *   exec - raw JS as string
 *   window_update - array('id', 'content', 'position')
 *      where position is: top, bottom, before, after.
 *      default is replace content.
 *   window_info - array('title', array('content'=>'content foo', 'childOf'=>'parent_name', array('button label'=>'action on button'[, 'other label'=>'action on button 2'])))
 *   window_create - array('id', 'title', array('option1_name'=>'option1_value))
 *   window_remove - id to remove as string
 *
 * Examples:
 *  (start code)
 *     $request[] = array('window_update', array('id1', 'Hello World', 'before'));
 *     $request[] = array('window_create', array('id2', 'Title of the window', 'HTML content', array('width'=>640, 'height'=>480)));
 *     $request[] = array('exec', 'alert("hello world");');
 *     echo os::buildRequest($request);
 *  (end)
 **********************************************/
    public function buildRequest($rows)
    {
        return @json_encode(array(
            'cols'=>array('type', 'data'),
            'rows'=>$rows)
        );
    }
	
/**********************************************
 * Method: getModuleConfig
 **********************************************/
	public function getModuleConfig()
	{
		$name = os::getModuleName();
		return parse_ini_file($name.'.ini');
	}

/**********************************************
 * Method: getModuleName
 **********************************************/
	public function getModuleName()
	{
		$file = basename($_SERVER['PHP_SELF']);
		return substr($file, 0, -4);
	}
	
/**********************************************
 * Method: getModulePath
 **********************************************/
	public function getModulePath()
	{
		$file = $_SERVER['PHP_SELF'];
		$file = explode('/', $file);
		array_pop($file);
		return implode('/', $file);
	}

/**********************************************
 * Method: getIcons
 * Build icons array
 *
 * Parameters:
 *      dirs - result of a readDir
 *      path - path to files
 *      parent - parent of icons
 *      container - container of icons
 *      dir_c - path in takos format (i.e.: /user/Desktop)
 **********************************************/
	public function getIcons($dirs, $path, $parent, $container, $dir_c='../../fs')
	{
		if(!is_array($dirs))
        {
            return false;
        }
        $out = array(
            'cols' => array('id', 'name', 'parent', 'container', 'icon', 'command', 'context', 'type'),
            'rows' => array()
        );
		$path_relative = os::lastSlash(str_replace($dir_c, '', $path));
		foreach($dirs as $k => $types) {
			if(count($types)) {
				foreach($types as $l => $dir) {
                    $dir_explorer = $path_relative.'/'.$dir;
					if($k==0) {
						$type = 'dir';
						$icon = 'themes/default/images/icons/folder.png';
						$command = 'OS.startApp(\'explorer\', {param:\''.$dir_explorer.'\'})';
						$context = 'console.log(\'context\')';
                        $type = 'dir';
					}
					else {
                        $type = 'file';
						$ext = strtolower(substr($dir, -4));
						if($ext=='.lnk')
						{
							$data = parse_ini_file($path.'/'.$dir);
							$type = 'file';
							$dir = $data['name'];
                            if(!$parent)
                            {
                                $parent = $data['parent'];
                            }
							$icon = $data['icon'];
							$command = $data['command'];
							$context = $data['context'];
						}
						elseif
						(
							$ext=='.jpg' OR
							$ext=='.jpeg' OR
							$ext=='.png' OR
							$ext=='.gif'
						)
						{
							$type = 'file';
							$icon = 'themes/default/images/icons/image.png';
							$command = 'OS.startApp(\'iviewer\', {param: \''.$dir_explorer.'\'})';
							$context = 'console.log(\'context\')';
						}
						else
						{
							$type = 'file';
							$icon = 'themes/default/images/icons/unknown.png';
							$command = 'OS.startApp(\'wordprocessor\', {param:\''.$dir.'\'})';
							$context = 'console.log(\'context\')';
						}
					}
					$out['rows'][] = array(
						$parent.'_'.$type.'_'.$dir,
						ucfirst($dir),
						$parent,
						$container,
						$icon,
						$command,
						$context,
                        $type
					);
				}
			}
		}
		return $out;
	}
}
?>
