/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

              v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: OS.widget.init.menu
 **********************************************/
OS.widget.init.menu = new Class({
    'Extends': OS.widget.base,
    'id': 'menu',
    'name': 'Programs',
    'options': {
        'orientation': 'north'
    },
    'elements': {
        'head': false,
        'body': false,
        'foot': false
    },
    
    /**********************************************
     * Method: create
     * Creation of the Menu
     **********************************************/
    'create': function()
    {
        var o = this;
        if($(this.id)){
            this.id = this.id+'_'+$time();
        }
        
        // Menu structure
        var container = new Element('ul', {
            'id': this.id,
            'class': 'menu'
        });
        this.elements.head = new Element('li', {
            'class': 'menu_head',
            'html': this.name,
            'events': {
                'click': function(e){
                    o.toggle();
                    e.stopPropagation();
                }
            }
        });
        this.elements.body = new Element('li', {
            'class': 'menu_body',
            'style': 'display:none'
        });
        this.elements.foot = new Element('li', {
            'class': 'menu_foot'
        });
        OS.os.addEvent('click', function(e){
            o.close(e);
        });
        
        // Create the menu
        this.elements.head.inject(container);
        this.elements.body.inject(container);
        this.elements.foot.inject(container);
        this.element = container;
        
        // Wait the modules to be fully loaded
        OS.showLoader(this.elements.head);
        var ready = setInterval(function(){
            if(OS.modules.loaded) {
                clearInterval(ready);
                o.update();
            }
        }, 100);
    },
    
    /**********************************************
     * Method: move
     **********************************************/
    'move': function(){
        var o = this;
        this.drag = new Drag.Move(this.element, {
            'container': this.options.container,
            'onStart': function(){
                o.element.setStyles({
                    'right': 'auto',
                    'bottom': 'auto'
                });
            }
        });
    },
    
    /**********************************************
     * Method: content
     * Create menu content (does not inject it)
     *
     * Returns:
     *  menu_content - List of programs for the menu as UL Element
     **********************************************/
    'content': function()
    {
        var menu_content = new Element('ul');
        var menu_elem, menu_child, menu_child_e, has_child;
        var o = this;
        if(OS.modules.info.length>0) {
            // Browse applications
            OS.modules.info.each(function(mod) {
                if(mod.childOf==='desktop') {
                    menu_elem = o.appLink(mod.id, mod.icon, mod.label);
                    menu_child = new Element('ul');
                    // Search Children
                    OS.modules.info.each(function(child) {
                        if(child.childOf===mod.id) {
                            menu_child_e = o.appLink(child.id, child.icon, child.label, 'child');
                            menu_child_e.inject(menu_child);
                            has_child=1;
                        }
                    });
                    if(has_child===1) {
                        menu_child.inject(menu_elem);
                        new Element('img', {
                            'src': 'themes/default/images/icons/tab_right.png',
                            'class': 'menu_parent'
                        }).inject(menu_elem);
                    }
                    menu_elem.inject(menu_content);
                }
            });
            return menu_content;
        }
        return false;
    },
    
    /**********************************************
     * Method: update
     * Recreate menu structure from modules list
     **********************************************/
    'update': function()
    {
        this.elements.body.empty();
        var content = this.content();
        if(content) {
            content.inject(this.elements.body);
        }
        OS.hideLoader(this.elements.head);
    },
    
    /**********************************************
     * Method: toggle
     * Toggle the menu in and out
     **********************************************/
    'toggle': function()
    {
        if(this.elements.body.style.display=='none') {
            this.open();
        }
        else {
            this.close();
        }
    },
    
    /**********************************************
     * Method: open
     * Open up the menu
     **********************************************/
    'open': function()
    {
        this.elements.body.style.display = 'block';
        this.checkOrientation();
    },
    
    /**********************************************
     * Method: close
     * Close down the menu
     **********************************************/
    'close': function()
    {
        this.elements.body.style.display = 'none';
        this.closeAllSubMenu(this.elements.body.getFirst('ul'));
    },
    
    /**********************************************
     * Method: appLink
     * Link to start an app in an LI element
     *
     * Parameters:
     *  id - id string of the Application
     *  icon - path to icon
     *  label - Display name of the application
     **********************************************/
    'appLink': function(id, icon, label, className)
    {
        className = className ? className : '';
        var o = this;
        var menu_elem = new Element('li', {
            'class': className,
            'html': label,
            'styles': {
                'background-image': 'url('+(icon?icon:'')+')',
                'background-position': '2px 50%',
                'background-repeat': 'no-repeat'
            },
            'events': {
                'mouseenter': function(e){
                    o.openSubMenu(menu_elem);
                },
                'click': function(e){
                    OS.startApp(id);
                    o.close();
                    e.stopPropagation();
                }
            }
        });
        return menu_elem;
    },
    
    /**********************************************
     * Method: openSubMenu
     **********************************************/
    'openSubMenu': function(elem)
    {
        this.closeAllSubMenu(elem.getParent());
        var child = elem.getFirst('ul');
        if(child){
            child.style.display = 'inline';
        }
    },
    
    /**********************************************
     * Method: closeSubMenu
     **********************************************/
    'closeSubMenu': function(elem)
    {
        var child= elem.getFirst('ul');
        if(child){
            child.style.display = 'none';
        }
    },
    
    /**********************************************
     * Method: closeAllSubMenu
     * FIXME: Close submenu of submenu
     **********************************************/
    'closeAllSubMenu': function(elem)
    {
        elem = elem || new Element;
        var o = this;
        var items = elem.getChildren('li');
        items.each(function(item){
            o.closeSubMenu(item);
        });
    },
    
    /**********************************************
     * Method: contextmenu
     **********************************************/
    'contextmenu': function(){
        var o = this;
        var menu = new OS.context({
            'container': o.elements.head,
            'list': [
                // Remove
                {
                    'name': 'Remove '+o.options.name,
                    'icon': 'themes/default/images/icons/cancel.png',
                    'action': function(){
                        OS.infoWindow('Remove '+o.options.name, 'remove_menu_question', {
                            'content': 'Are you sure you want to remove <b>'+o.options.name+'</b> ?',
                            'actions': {
                                'No': function(){},
                                'Yes': function(){o.remove();}
                            },
                            'modal': false
                        });
                    }
                },
                // Properties
                {
                    'name': 'Properties',
                    'icon': 'themes/default/images/icons/package_system.png',
                    'action': function(){
                        o.properties();
                    }
                }
            ]
        });
    },
    
    /**********************************************
     * Method: properties
     **********************************************/
    'properties': function(elem)
    {
        var id = this.id+'_properties';
        OS.createWindow(this.options.name+' Properties', id,
        {
            'width': 400,
            'height': 300,
            'resizable': true,
            'onCreated': function()
            {
                var content = $(id+'_content');
                
                // Options
                t = $time();
                var options = new OS.panel();
                options.addElement('button', t, 'Apply Changes', {
                    'className': 'programs_btn_apply',
                    'action': function() {
                    }
                });
                
                options.inject(content);
            }
        });
    },
    
    /**********************************************
     * Method: setOrientation
     * Position the menu with options.orientation
     **********************************************/
    'setOrientation': function()
    {
        var head_size = this.elements.head.getSize();
        var body = this.elements.body, pos, orientation;
        body.setStyles({
            'right': '',
            'left': '',
            'top': '',
            'bottom': ''
        });
        switch(this.options.orientation) {
            case 'west':
                body.setStyles({
                    'right': head_size.x+'px',
                    'top': 0
                });
                break;
            case 'east':
                body.setStyles({
                    'left': head_size.x+'px',
                    'top': 0
                });
                break;
            case 'south':
                body.setStyles({
                    'top': head_size.y+'px',
                    'left': 0
                });
                break;
            case 'north':
            default:
                body.setStyles({
                    'bottom': head_size.y+'px',
                    'left': 0
                });
        }
    },
    
    /**********************************************
     * Method: checkOrientation
     * Check what is current orientation and set options.orientation
     **********************************************/
    'checkOrientation': function(i)
    {
        var body_pos = this.elements.body.getPosition();
        var body_size = this.elements.body.getSize();
        var dw = OS.dw, dh = OS.dh;
        if(
            body_pos.x<0 ||
            body_pos.y<0 ||
            body_pos.x+body_size.x>window.getSize().x ||
            body_pos.y+body_size.y>dh
        ) {
            switch(this.options.orientation) {
                case 'north':
                        this.options.orientation='east';
                    break;
                case 'east':
                        this.options.orientation='south';
                    break;
                case 'south':
                        this.options.orientation='west';
                    break;
                case 'west':
                        this.options.orientation='north';
                    break;
            }
            this.setOrientation();
            
            i=i||0;
            i+=1;
            if(i<4){
                this.checkOrientation(i);
            }
        }
    }
});
