/*
  _______    _      ____   _____ 
 |__   __|  | |    / __ \ / ____|
    | | __ _| | __| |  | | (___  
    | |/ _` | |/ /| |  | |\___ \ 
    | | (_| |   < | |__| |____) |
    |_|\__,_|_|\_\ \____/|_____/

            v. 1.3
        Web Operating System
*/
/**********************************************
 * Class: programs
 * Applications management
 **********************************************/
OS.modules.init.push({
	'id': 'programs',
	'init': new Class({

'label': 'Programs',
'childOf': 'admin',
'path': 'programs/admin/programs.php',
'output': {},
'homeText': '   ____ ___  ___  ____\r\n   |__| |__] |__] [__  \r\n   |  | |    |    ___] \r\n     MANAGEMENT 1.0',
'minx': 420,
'miny': 235,

/**********************************************
 * Method: start
 **********************************************/
'start': function()
{
	OS.createWindow(this.label, this.id,
	{
		'width': this.width,
		'height': this.height,
        'minx': this.minx,
        'miny': this.miny,
        'statusbar': false,
        'onCreated': function()
        {
            var id = OS.modules.run.programs.id;
            var content = $(id+'_content');
            var t, checked, tips;
            
            // Programs
            var programs = new OS.panel({
                'orientation': 'west',
                'size': 200,
                'padding': 3,
                'resizable': true
            });
            OS.modules.list.each(function(mod, i)
            {
                if(mod.id)
                {
                    t = $time()+'_'+$random(0, 100);
                    checked = mod.state=='ii' ? 'checked' : '';
                    programs.addElement('checkbox', t, mod.label, {'checked': checked, 'class': 'programs_tips'});
                }
            });
            
            // Options
            t = $time();
			var options = new OS.panel({
                'orientation': 'north',
                'size': 135
            });
            options.addElement('button', t, 'Apply Changes', {
                'className': 'programs_btn_apply',
                'action': function() {
                    OS.modules.run.programs.apply();
                }
            });
            options.addElement('button', t, 'Select All', {'action': function() {
                OS.modules.run.programs.selectAll(true);
            }});
            options.addElement('button', t, 'Unselect All', {'action': function() {
                OS.modules.run.programs.selectAll(false);
            }});
            options.addElement('button', t, 'Reset', {'action': function() {
                OS.modules.run.programs.reset();
            }});
            options.addElement('button', t, 'Rebuild list', {'action': function() {
                OS.buildModulesList(false, id);
            }});
            
            // Output
			var output = new OS.panel({
                'orientation': 'south',
                'separate': false
            });
            output.addElement('pre', t);
            
            programs.inject(content);
            options.inject(content);
            output.inject(content);
            
			OS.modules.run.programs.programs = programs.elements.box;
			OS.modules.run.programs.output = $(t+'_gui_pre');
            OS.modules.run.programs.output.set('html', OS.modules.run.programs.homeText);
        }
	});
},

/**********************************************
 * Method: getBoxes
 * Return checkboxes dictionnary with their status
 **********************************************/
'getBoxes': function()
{
    var boxes=[];
    var checked, id, cb;
	OS.modules.run.programs.programs.getChildren().each(function(i, j)
	{
		cb = i.getChildren('.checkbox');
		id = cb.get('id')[0];
		checked = cb.get('checked')[0];
        boxes.push({'id': id, 'checked': checked});
	});
    return boxes;
},

/**********************************************
 * Method: selectAll
 * Select or unselect all checkboxes
 **********************************************/
'selectAll': function(checked)
{
    checked = checked ? 'checked' : '';
    var boxes=[];
    var cb;
	OS.modules.run.programs.programs.getChildren().each(function(i, j)
	{
		cb = i.getChildren('.checkbox');
        cb.set('checked', checked);
	});
    return boxes;
},

/**********************************************
 * Method: reset
 * Reset checkboxes where it was at last save
 **********************************************/
'reset': function()
{
    var t, checked;
	OS.modules.run.programs.programs.getChildren().each(function(i, j)
	{
		cb = i.getChildren('.checkbox');
        if(OS.modules.list[j].state=='ii')
        {
            cb.set('checked', 'checked');
        }
        else
        {
            cb.set('checked', '');
        }
	});
    this.output.set('html', this.homeText);
},

/**********************************************
 * Method: install
 * Send a request to copy new apps states to
 * Kernel/cnf/applications file.
 **********************************************/
'install': function()
{
    OS.sendRequest(OS.modules.run.programs.path, {
        'post': 'req=install&apps='+OS.dbJson(['id', 'label', 'path', 'state'], OS.modules.list)
    });
},

/**********************************************
 * Method: apply
 * Apply apps states in OS.modules.list and call install()
 **********************************************/
'apply': function()
{
	var cb, id, checked, label, text;
	var n_new = 0;
	var n_remove = 0;
	var n_install = 0;
    var boxes = this.getBoxes();
    var out = this.output;
    out.set('html', '');
    text = ' __CHANGES__________________\r\n|\r\n';
    boxes.each(function(box, i)
    {
        label = OS.modules.list[i].label;
        if(box.checked) {
            if(OS.modules.list[i].state!='ii') {
                text += '| install '+label+'\r\n';
                n_new+=1;
            }
            n_install+=1;
        }
        else {
            if(OS.modules.list[i].state=='ii') {
                text += '| remove '+label+'\r\n';
                n_remove+=1;
            }
        }
    });
    text += '|___________________________\r\n';
    if(n_new || n_remove)
    {
        out.appendText(text);
    }
    else
    {
        out.appendText('--- Nothing to do ---');
        setTimeout(function(){
            OS.modules.run.programs.output.set('html', OS.modules.run.programs.homeText);
        }, 2000);
        return false;
    }

	var out = this.output;
    var install = this.install;
    OS.infoWindow('Alert !', 'menu_CloseConfirm', {
        'content': 'Do you want to <strong>Install All Selected Apps</strong> ?',
        'childOf': 'programs',
        'actions': {
            'yes': function()
            {
                boxes.each(function(box, i) {
                    label = OS.modules.list[i].label;
                    if(box.checked) {
                        if(OS.modules.list[i].state!='ii') {
                            OS.modules.list[i].state = 'ii';
                        }
                    }
                    else {
                        if(OS.modules.list[i].state=='ii') {
                            OS.modules.list[i].state = 'rc';
                        }
                    }
                });
                install();
                out.appendText('|___________________________\r\n\r\n', 'top');
                out.appendText('| '+n_install+' installed in total\r\n', 'top');
                out.appendText('| '+n_remove+' removed\r\n', 'top');
                out.appendText('| '+n_new+' new\r\n', 'top');
                out.appendText('| '+OS.modules.list.length+' Applications available.\r\n', 'top');
                out.appendText('\r\n __DONE_____________________\r\n|\r\n', 'top');
            },
            'no': function(){
                out.set('html', OS.modules.run.programs.homeText);
            }
        }
    });
    return true;
}

})});
